Package: GeoTcgaData
Type: Package
Title: Processing Various Types of Data on GEO and TCGA
Version: 2.12.0
Authors@R: person(given = "Erqiang", family = "Hu", email = "13766876214@163.com", role  = c("aut", "cre"), comment = c(ORCID = "0000-0002-1798-7513"))
Description: Gene Expression Omnibus(GEO) and The Cancer Genome Atlas
        (TCGA) provide us with a wealth of data, such as RNA-seq, DNA
        Methylation, SNP and Copy number variation data. It's easy to
        download data from TCGA using the gdc tool, but processing
        these data into a format suitable for bioinformatics analysis
        requires more work. This R package was developed to handle
        these data.
Depends: R (>= 4.2.0)
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.2.3
Suggests: knitr, rmarkdown, DESeq2, S4Vectors, ChAMP, impute, tidyr,
        clusterProfiler, org.Hs.eg.db, edgeR, limma, quantreg, minfi,
        IlluminaHumanMethylation450kanno.ilmn12.hg19, dearseq, NOISeq,
        testthat (>= 3.0.0), CATT, TCGAbiolinks, enrichplot, GEOquery,
        BiocGenerics
VignetteBuilder: knitr
Imports: utils, data.table, plyr, cqn, topconfects, stats,
        SummarizedExperiment, methods
Language: en-US
URL: https://github.com/YuLab-SMU/GeoTcgaData
BugReports: https://github.com/YuLab-SMU/GeoTcgaData/issues
biocViews: GeneExpression, DifferentialExpression, RNASeq,
        CopyNumberVariation, Microarray, Software, DNAMethylation,
        DifferentialMethylation, SNP, ATACSeq, MethylationArray
Config/testthat/edition: 3
Config/pak/sysreqs: zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 13:00:16 UTC
RemoteUrl: https://github.com/bioc/GeoTcgaData
RemoteRef: RELEASE_3_23
RemoteSha: 4738d3e067f9f1ffd220e5ae50a931b31aa382e7
NeedsCompilation: no
Packaged: 2026-04-29 22:32:28 UTC; root
Author: Erqiang Hu [aut, cre] (ORCID: <https://orcid.org/0000-0002-1798-7513>)
Maintainer: Erqiang Hu <13766876214@163.com>
Built: R 4.6.0; ; 2026-04-30 00:25:19 UTC; unix

# Generated by roxygen2: do not edit by hand

export(Merge_methy_tcga)
export(SNP_QC)
export(array_preprocess)
export(cal_mean_module)
export(cluster_array)
export(combine_pvalue)
export(countToFpkm)
export(countToTpm)
export(differential_CNV)
export(differential_RNA)
export(differential_SNP)
export(differential_SNP_GEO)
export(differential_SNP_tcga)
export(differential_array)
export(differential_limma)
export(differential_methy)
export(fpkmToTpm)
export(gene_ave)
export(get_geo_array)
export(id_conversion_TCGA)
export(prepare_chi)
export(repAssign)
export(repRemove)
import(cqn)
import(methods)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(plyr,rename)
importFrom(stats,as.dist)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,hclust)
importFrom(stats,na.fail)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(utils,methods)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# GeoTcgaData

The goal of GeoTcgaData is to deal with RNA-seq, DNA Methylation, single
nucleotide Variation and Copy number variation data in GEO and TCGA.

## :writing_hand: Authors

Erqiang Hu

Department of Bioinformatics, School of Basic Medical Sciences, Southern
Medical University.

## :arrow_double_down: Installation

``` r
if(!requireNamespace("devtools", quietly = TRUE))
    install.packages("devtools")
devtools::install_github("YuLab-SMU/GeoTcgaData")
```

``` r
library(GeoTcgaData)
```

GEO and TCGA provide us with a wealth of data, such as RNA-seq, DNA
Methylation, single nucleotide Variation and Copy number variation data.
It’s easy to download data from TCGA using the gdc tool or
`TCGAbiolinks`, and some software provides organized TCGA data, such as
[UCSC Xena](http://xena.ucsc.edu/) ,
[UCSCXenaTools](https://cran.r-project.org/package=UCSCXenaTools)，and
[sangerbox](http://vip.sangerbox.com/), but processing these data into a
format suitable for bioinformatics analysis requires more work. This R
package was developed to handle these data.

## Example

This is a basic example which shows you how to solve a common problem:

## RNA-seq data differential expression analysis

It is convenient to use
[`TCGAbiolinks`](http://www.bioconductor.org/packages/release/bioc/vignettes/TCGAbiolinks/inst/doc/analysis.html)
or [`GDCRNATools`](https://bioconductor.org/packages/GDCRNATools/) to
download and analysis Gene expression data. `TCGAbiolinks` use `edgeR`
package to do differential expression analysis, while `GDCRNATools` can
implement three most commonly used methods: limma, edgeR , and DESeq2 to
identify differentially expressed genes (DEGs).

Alicia Oshlack et. al. claimed that unlike the chip data, the RNA-seq
data had one [bias](https://pubmed.ncbi.nlm.nih.gov/20132535/): the
larger the transcript length / mean read count , the more likely it was
to be identified as a differential gene, [while there was no such trend
in the chip data](https://pubmed.ncbi.nlm.nih.gov/19371405/).

However, when we use their chip data for difference analysis( using the
limma package), we find that chip data has the same trend as RNA-seq
data. And we also found this trend in the difference analysis results
given by the data
[authors](https://genome.cshlp.org/content/18/9/1509.long).

It is worse noting that [only technical replicate data, which has small
gene dispersions, shows this
bias](https://pubmed.ncbi.nlm.nih.gov/28545404/). This is because in
technical replicate RNA-seq data a long gene has more reads mapping to
it compared to a short gene of similar expression, and most of the
statistical methods used to detect differential expression have stronger
detection ability for genes with more reads. However, we have not
deduced why there is such a bias in the current difference analysis
algorithms.

Some software, such as [CQN](http://www.bioconductor.org/packages/cqn/)
, present a [normalization
algorithm](https://pubmed.ncbi.nlm.nih.gov/22285995/) to correct
systematic biases(gene length bias and [GC-content
bias](https://pubmed.ncbi.nlm.nih.gov/22177264/). But they did not
provide sufficient evidence to prove that the correction is effective.
We use the [Marioni dataset](https://pubmed.ncbi.nlm.nih.gov/19371405/)
to verify the correction effect of CQN and find that there is still a
deviation after correction:

[GOseq](http://bioconductor.org/packages/goseq/) based on [Wallenius’
noncentral hypergeometric
distribution](https://en.wikipedia.org/wiki/Wallenius%27_noncentral_hypergeometric_distribution)
can effectively correct the gene length deviation in enrichment
analysis. However, the current RNA-seq data often have no gene length
bias, but only the expression amount(read count) bias, GOseq may
overcorrect these data, correcting originally unbiased data into reverse
bias.

GOseq also fails to correct for expression bias, therefore, read count
bias correction is still a challenge for us.

use `TCGAbiolinks` to download TCGA data

``` r
# download RNA-seq data
library(TCGAbiolinks)
                
query <- GDCquery(project = "TCGA-ACC",
                  data.category = "Transcriptome Profiling",
                  data.type = "Gene Expression Quantification", 
                  workflow.type = "STAR - Counts")
                  
GDCdownload(query, method = "api", files.per.chunk = 3, 
    directory = Your_Path)

dataRNA <- GDCprepare(query = query, directory = Your_Path,
                      save = TRUE, save.filename = "dataRNA.RData")
## get raw count matrix                         
dataPrep <- TCGAanalyze_Preprocessing(object = dataRNA,
                                      cor.cut = 0.6,
                                      datatype = "STAR - Counts")
```

Use `differential_RNA` to do difference analysis. We provide the data of
human gene length and GC content in `gene_cov`.

``` r
group <- sample(c("grp1", "grp2"), ncol(dataPrep), replace = TRUE)
library(cqn) # To avoid reporting errors: there is no function "rq"
## get gene length and GC content
library(org.Hs.eg.db)
genes_bitr <- bitr(rownames(gene_cov), fromType = "ENTREZID", toType = "ENSEMBL", 
         OrgDb = org.Hs.eg.db, drop = TRUE)
genes_bitr <- genes_bitr[!duplicated(genes_bitr[,2]), ]
gene_cov2 <- gene_cov[genes_bitr$ENTREZID, ]
rownames(gene_cov2) <- genes_bitr$ENSEMBL
genes <- intersect(rownames(dataPrep), rownames(gene_cov2))
dataPrep <- dataPrep[genes, ]
geneLength <- gene_cov2[genes, "length"]
gccontent <- gene_cov2[genes, "GC"]
names(geneLength) <- names(gccontent) <- genes
##  Difference analysis
DEGAll <- differential_RNA(counts = dataPrep, group = group, 
                   geneLength = geneLength, gccontent = gccontent)
```

Use `clusterProfiler` to do enrichment analytics:

``` r
diffGenes <- DEGAll$logFC
names(diffGenes) <- rownames(DEGAll)
diffGenes <- sort(diffGenes, decreasing = TRUE)
library(clusterProfiler)
library(enrichplot)
library(org.Hs.eg.db)
gsego <- gseGO(gene = diffGenes, OrgDb = org.Hs.eg.db, keyType = "ENSEMBL")
dotplot(gsego)
```

## DNA Methylation data integration

use `TCGAbiolinks` to download TCGA data.

The codes may need to be modified if `TCGAbiolinks` updates. So please
read its
[documents](https://www.bioconductor.org/packages/release/bioc/html/TCGAbiolinks.html).

``` r
library(TCGAbiolinks)
query <- GDCquery(project = "TCGA-ACC",
                  data.category = "DNA Methylation",
                  data.type = "Methylation Beta Value",
                  platform = "Illumina Human Methylation 450")
GDCdownload(query, method = "api", files.per.chunk = 5, directory = Your_Path)
```

The function `Merge_methy_tcga` could Merge methylation data downloaded
from TCGA official website or TCGAbiolinks. This makes it easier to
extract differentially methylated genes in the downstream analysis. For
example:

``` r
merge_result <- Merge_methy_tcga(Your_Path_to_DNA_Methylation_data)
```

Then use differential_methy() to do difference analysis.

``` r
# if (!requireNamespace("ChAMP", quietly = TRUE))
#     BiocManager::install("ChAMP")
library(ChAMP) # To avoid reporting errors
differential_gene <- differential_methy(cpgData = merge_result, sampleGroup = sample(c("C","T"), 
    ncol(merge_result[[1]]), replace = TRUE))
```

**Note:** `ChAMP`has a large number of dependent packages. If you cannot
install it successfully, you can download each dependent package
separately(Source or Binary) and install it locally.

If your methylation data was downloaded from [UCSC
Xena](http://xena.ucsc.edu/), you can use
`differential_methy(ucscData = TRUE)` to get differential genes.

``` r
methy_file <- "TCGA.THCA.sampleMap_HumanMethylation450.gz"
methy <- fread(methy_file, sep = "\t", header = T)
library(ChAMP)
myImport <- champ.import(directory=system.file("extdata",package="ChAMPdata"))
myfilter <- champ.filter(beta=myImport$beta,pd=myImport$pd,detP=myImport$detP,beadcount=myImport$beadcount)
cpg_gene <- hm450.manifest.hg19[, c("probeID", "gene_HGNC")]
## or use IlluminaHumanMethylation450kanno.ilmn12.hg19 to get annotation data
# library(IlluminaHumanMethylation450kanno.ilmn12.hg19)
# ann <- getAnnotation(IlluminaHumanMethylation450kanno.ilmn12.hg19)
# class(ann) <- "data.frame"
# cpg_gene <- ann[,c("Name", "UCSC_RefGene_Name", "UCSC_RefGene_Group")]

methy_df <- differential_methy(methy, cpg_gene, ucscData = TRUE)
```

We provide three models to get methylation difference genes:

if model = “cpg”, step1: calculate difference cpgs; step2: calculate
difference genes;

if model = “gene”, step1: calculate the methylation level of genes;
step2: calculate difference genes.

We find that only model = “gene” has no deviation of CpG number.

Use `clusterProfiler` to do enrichment analytics:

``` r
differential_gene$p.adj <- p.adjust(differential_gene$pvalue)
genes <- differential_gene[differential_gene$p.adj < 0.05, "gene"]
library(clusterProfiler)
library(enrichplot)
library(org.Hs.eg.db)
ego <- enrichGO(gene = genes, OrgDb = org.Hs.eg.db, keyType = "SYMBOL")
dotplot(ego)
```

## Copy number variation data integration and differential gene extraction

use TCGAbiolinks to download TCGA data(Gene Level Copy Number Scores)

``` r
library(TCGAbiolinks)
query <- GDCquery(project = "TCGA-LGG",
                  data.category = "Copy Number Variation",
                  data.type = "Gene Level Copy Number Scores")

GDCdownload(query, method = "api", files.per.chunk = 5, directory = Your_Path)

data <- GDCprepare(query = query, 
                   directory =  Your_Path) 
```

Do difference analysis of gene level copy number variation data using
`differential_CNV`

``` r
class(data) <- "data.frame"
cnvData <- data[, -c(1,2,3)]
rownames(cnvData) <- data[, 1]
sampleGroup  = sample(c("A","B"), ncol(cnvData), replace = TRUE)
diffCnv <- differential_CNV(cnvData, sampleGroup)
```

Use `clusterProfiler` to do enrichment analytics:

``` r
pvalues <- diffCnv$pvalue * sign(diffCnv$odds)
genes <- rownames(diffCnv)[diffCnv$pvalue < 0.05]
library(clusterProfiler)
library(enrichplot)
library(org.Hs.eg.db)
ego <- enrichGO(gene = genes, OrgDb = org.Hs.eg.db, keyType = "ENSEMBL")
dotplot(ego)
```

## Difference analysis of single nucleotide Variation data

Use TCGAbiolinks to download TCGA data

``` r
library(TCGAbiolinks)
query <- GDCquery(project = "TCGA-ACC",
                  data.category = "Simple Nucleotide Variation",
                  data.type = "Masked Somatic Mutation",
                  workflow.type = "MuSE Variant Aggregation and Masking")

GDCdownload(query, method = "api", files.per.chunk = 5, directory = Your_Path)

data_snp <- GDCprepare(query = query, 
                   directory =  Your_Path) 
```

Use `differential_SNP_tcga` to do difference analysis

``` r
samples <- unique(data_snp$Tumor_Sample_Barcode)
sampleType <- sample(c("A","B"), length(samples), replace = TRUE)
names(sampleType) <- samples
pvalue <- differential_SNP_tcga(snpData = data_snp, sampleType = sampleType)
# merge pvalue
```

Use `clusterProfiler` to do enrichment analysis

``` r
pvalue2 <- sort(pvalue, decreasing = TRUE)
library(clusterProfiler)
library(enrichplot)
library(org.Hs.eg.db)
gsego <- gseGO(pvalue2, OrgDb = org.Hs.eg.db, keyType = "SYMBOL")
dotplot(gsego)
```

## GEO chip data processing

The function `gene_ave` could average the expression data of different
ids for the same gene in the GEO chip data. For example:

``` r
aa <- c("MARCH1","MARC1","MARCH1","MARCH1","MARCH1")
bb <- c(2.969058399,4.722410064,8.165514853,8.24243893,8.60815086)
cc <- c(3.969058399,5.722410064,7.165514853,6.24243893,7.60815086)
file_gene_ave <- data.frame(aa=aa,bb=bb,cc=cc)
colnames(file_gene_ave) <- c("Gene", "GSM1629982", "GSM1629983")
result <- gene_ave(file_gene_ave, 1)
```

Multiple genes symbols may correspond to a same chip id. The result of
function `repAssign` is to assign the expression of this id to each
gene, and function `repRemove` deletes the expression. For example:

``` r
aa <- c("MARCH1 /// MMA","MARC1","MARCH2 /// MARCH3","MARCH3 /// MARCH4","MARCH1")
bb <- c("2.969058399","4.722410064","8.165514853","8.24243893","8.60815086")
cc <- c("3.969058399","5.722410064","7.165514853","6.24243893","7.60815086")
input_file <- data.frame(aa=aa,bb=bb,cc=cc)

repAssign_result <- repAssign(input_file," /// ")
repRemove_result <- repRemove(input_file," /// ")
```

## Other downstream analyses

1.  Especially, the function id_conversion could convert ENSEMBL gene id
    to gene Symbol in TCGA. For example:

``` r
data(profile)
result <- id_conversion_TCGA(profile)
#> 
#> 
#> 'select()' returned 1:1 mapping between keys and columns
#> Warning in clusterProfiler::bitr(rownames(profiles), fromType = "ENSEMBL", :
#> 22.22% of input gene IDs are fail to map...
```

The parameter profile is a data.frame or matrix of gene expression data
in TCGA.

**Note:** In previous versions(\< 1.0.0) the `id_conversion` and
`id_conversion` used HGNC data to convert human gene id. In future
versions, we will use `clusterProfiler::bitr` for ID conversion.

``` r
library(clusterProfiler)
#> clusterProfiler v4.10.0  For help: https://yulab-smu.top/biomedical-knowledge-mining-book/
#> 
#> If you use clusterProfiler in published research, please cite:
#> T Wu, E Hu, S Xu, M Chen, P Guo, Z Dai, T Feng, L Zhou, W Tang, L Zhan, X Fu, S Liu, X Bo, and G Yu. clusterProfiler 4.0: A universal enrichment tool for interpreting omics data. The Innovation. 2021, 2(3):100141
#> 
#> 载入程辑包：'clusterProfiler'
#> The following object is masked from 'package:stats':
#> 
#>     filter
library(org.Hs.eg.db)
#> 载入需要的程辑包：AnnotationDbi
#> 载入需要的程辑包：stats4
#> 载入需要的程辑包：BiocGenerics
#> 
#> 载入程辑包：'BiocGenerics'
#> The following objects are masked from 'package:stats':
#> 
#>     IQR, mad, sd, var, xtabs
#> The following objects are masked from 'package:base':
#> 
#>     anyDuplicated, aperm, append, as.data.frame, basename, cbind,
#>     colnames, dirname, do.call, duplicated, eval, evalq, Filter, Find,
#>     get, grep, grepl, intersect, is.unsorted, lapply, Map, mapply,
#>     match, mget, order, paste, pmax, pmax.int, pmin, pmin.int,
#>     Position, rank, rbind, Reduce, rownames, sapply, setdiff, sort,
#>     table, tapply, union, unique, unsplit, which.max, which.min
#> 载入需要的程辑包：Biobase
#> Welcome to Bioconductor
#> 
#>     Vignettes contain introductory material; view with
#>     'browseVignettes()'. To cite Bioconductor, see
#>     'citation("Biobase")', and for packages 'citation("pkgname")'.
#> 载入需要的程辑包：IRanges
#> 载入需要的程辑包：S4Vectors
#> 
#> 载入程辑包：'S4Vectors'
#> The following object is masked from 'package:clusterProfiler':
#> 
#>     rename
#> The following object is masked from 'package:utils':
#> 
#>     findMatches
#> The following objects are masked from 'package:base':
#> 
#>     expand.grid, I, unname
#> 
#> 载入程辑包：'IRanges'
#> The following object is masked from 'package:clusterProfiler':
#> 
#>     slice
#> The following object is masked from 'package:grDevices':
#> 
#>     windows
#> 
#> 载入程辑包：'AnnotationDbi'
#> The following object is masked from 'package:clusterProfiler':
#> 
#>     select
bitr(c("A2ML1", "A2ML1-AS1", "A4GALT", "A12M1", "AAAS"), fromType = "SYMBOL", 
     toType = "ENSEMBL", OrgDb = org.Hs.eg.db, drop = FALSE)
#> 'select()' returned 1:many mapping between keys and columns
#> Warning in bitr(c("A2ML1", "A2ML1-AS1", "A4GALT", "A12M1", "AAAS"), fromType =
#> "SYMBOL", : 40% of input gene IDs are fail to map...
#>      SYMBOL         ENSEMBL
#> 1     A2ML1 ENSG00000166535
#> 2 A2ML1-AS1            <NA>
#> 3    A4GALT ENSG00000128274
#> 4     A12M1            <NA>
#> 5      AAAS ENSG00000094914
#> 6      AAAS ENSG00000291836
```

2.  The function `countToFpkm` and `countToTpm` could convert count data
    to FPKM or TPM data.

``` r
data(gene_cov)
lung_squ_count2 <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), ncol = 3)
rownames(lung_squ_count2) <- c("DISC1", "TCOF1", "SPPL3")
colnames(lung_squ_count2) <- c("sample1", "sample2", "sample3")
result <- countToFpkm(lung_squ_count2,
  keyType = "SYMBOL",
  gene_cov = gene_cov
)
#> 'select()' returned 1:1 mapping between keys and columns
#> Warning in clusterProfiler::bitr(rownames(gene_cov), fromType = "ENTREZID", :
#> 0.07% of input gene IDs are fail to map...
result
#>        sample1  sample2  sample3
#> DISC1 11449.25 18318.79 20036.18
#> TCOF1 29140.08 29140.08 29140.08
#> SPPL3 69473.39 55578.71 52105.04
```

``` r
lung_squ_count2 <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), ncol = 3)
rownames(lung_squ_count2) <- c("DISC1", "TCOF1", "SPPL3")
colnames(lung_squ_count2) <- c("sample1", "sample2", "sample3")
result <- countToTpm(lung_squ_count2,
  keyType = "SYMBOL",
  gene_cov = gene_cov
)
#> 'select()' returned 1:1 mapping between keys and columns
#> Warning in clusterProfiler::bitr(rownames(gene_cov), fromType = "ENTREZID", :
#> 0.07% of input gene IDs are fail to map...
result
#>        sample1  sample2  sample3
#> DISC1 104024.7 177787.5 197827.0
#> TCOF1 264758.8 282810.2 287714.3
#> SPPL3 631216.4 539402.3 514458.7
```

**Note:** Now the combined clinical data can be downloaded directly from
[TCGAbiolinks](http://www.bioconductor.org/packages/release/bioc/vignettes/TCGAbiolinks/inst/doc/clinical.html).

``` r
library(TCGAbiolinks)
## get BCR Biotab data
query <- GDCquery(project = "TCGA-ACC", 
                  data.category = "Clinical",
                  data.type = "Clinical Supplement", 
                  data.format = "BCR Biotab")
GDCdownload(query)
clinical.BCRtab.all <- GDCprepare(query)
names(clinical.BCRtab.all)

## get indexed data
clinical <- GDCquery_clinic(project = "TCGA-ACC", type = "clinical")
```

#setwd("E:\\GeoTcgaData_work")
hgnc_file <- data.table::fread("E:\\GeoTcgaData_work\\hgnc_complete_set.txt", 
  sep = "\t", header = TRUE)
hgnc_file <- dplyr::select(hgnc_file, -c("alias_symbol", "alias_name", 
  "prev_symbol", "lsdb", "agr"))
class(hgnc_file) <- "data.frame"
gene_loc_len <- GeoTcgaData:::gene_loc_len
hgnc <- GeoTcgaData:::hgnc
genePos <- GeoTcgaData:::genePos
hgnc_file <- GeoTcgaData:::hgnc_file
usethis::use_data(hgnc_file, hgnc, gene_loc_len, 
  internal = TRUE, compress = "xz", overwrite = TRUE)


## gene_cov
library(TxDb.Hsapiens.UCSC.hg38.knownGene)
library(BSgenome.Hsapiens.UCSC.hg38)
hg38_TxDb <- TxDb.Hsapiens.UCSC.hg38.knownGene
hg38 <- BSgenome.Hsapiens.UCSC.hg38
calc_gene_cov <- function(TxDB, BSGENOME){
    Gene <- genes(TxDB, single.strand.genes.only = FALSE)
    Exon <- exons(x = TxDB)
    Overlap <- findOverlaps(Exon, Gene)
    Exon <- Exon[queryHits(Overlap)]
    mcols(Exon)$gene_id <- mcols(Gene[subjectHits(Overlap)])$gene_id
    Exon <- split(Exon, mcols(Exon)$gene_id)
    Exon <- reduce(Exon)
    calculate_cov <- function(x){
        xlen <- sum(width(x))
        xseq <- BSgenome::getSeq(BSGENOME, x)
        xGC <- sum(Biostrings::letterFrequency(xseq, 'GC'))/xlen
        c(xlen, xGC)
    }
    gene_cov <- lapply(Exon, calculate_cov)
    gene_cov <- gene_cov[names(Gene)]
    gene_cov <- t(as.data.frame(gene_cov))
    rownames(gene_cov) <- names(Gene)
    colnames(gene_cov) <- c('length', 'GC')
    as.data.frame(gene_cov)
}
gene_cov <- calc_gene_cov(TxDB = hg38_TxDb, BSGENOME = hg38)

# copy from metap package: https://CRAN.R-project.org/package=metap
#' @importFrom stats na.fail
#' @importFrom stats pchisq
#' @importFrom stats pnorm
#' @importFrom stats qnorm
sumz <- function(p, weights = NULL, data = NULL, subset = NULL,
   na.action = na.fail, log.p = FALSE, log.input = FALSE)  {
   if(is.null(data)) data <- sys.frame(sys.parent())
   mf <- match.call()
   mf$data <- NULL
   mf$subset <- NULL
   mf$na.action <- NULL
   mf[[1]] <- as.name("data.frame")
   mf <- eval(mf, data)
   if(!is.null(subset)) mf <- mf[subset,]
   mf <- na.action(mf)
   p <- as.numeric(mf$p)
   weights <- mf$weights
   noweights <- is.null(weights)
   if(noweights) weights <- rep(1, length(p))
   if(length(p) != length(weights)) warning("Length of p and weights differ")
   if(log.input) {
      keep <- p < 0
   } else {
      keep <- (p > 0) & (p < 1)
   }
   invalid <- sum(1L * keep) < 2
   if(invalid) {
      warning("Must have at least two valid p values")
      res <- list(z = NA_real_, p = NA_real_,
         validp = p[keep], weights = weights)
   } else {
      if(sum(1L * keep) != length(p)) {
         warning("Some studies omitted")
         omitw <- weights[!keep]
         if((sum(1L * omitw) > 0) & !noweights)
            warning("Weights omitted too")
      }
      zp <- (qnorm(p[keep], lower.tail = FALSE, log.p = log.input) %*%
         weights[keep]) / sqrt(sum(weights[keep]^2))
      res <- list(z = zp, p = pnorm(zp, lower.tail = FALSE,
            log.p = log.p),
         validp = p[keep], weights = weights)
   }
   class(res) <- c("sumz", "metap")
   res
}

# copy from metap package: https://CRAN.R-project.org/package=metap
print.sumz <- function(x, ...) {
   cat("sumz = ", x$z, "p = ", x$p, "\n")
   invisible(x)
}

# copy from metap package: https://CRAN.R-project.org/package=metap
sumlog <- function(p, log.p = FALSE) {
   keep <- (p > 0) & (p <= 1)
   invalid <- sum(1L * keep) < 2
   if(invalid) {
      warning("Must have at least two valid p values")
      res <- list(chisq = NA_real_, df = NA_integer_,
         p = NA_real_, validp = p[keep])
   } else {
      lnp <- log(p[keep])
      chisq <- (-2) * sum(lnp)
      df <- 2 * length(lnp)
      if(length(lnp) != length(p)) {
         warning("Some studies omitted")
      }
      res <- list(chisq = chisq, df = df,
         p = pchisq(chisq, df, lower.tail = FALSE,
            log.p = log.p), validp = p[keep])
    }
   class(res) <- c("sumlog", "metap")
   res
}

# copy from metap package: https://CRAN.R-project.org/package=metap
print.sumlog <- function(x, ...) {
   cat("chisq = ", x$chisq, " with df = ", x$df, " p = ", x$p, "\n")
   invisible(x)
}

#' Find the mean value of the gene in each module
#'
#' @param geneExpress a data.frame of gene expression data. 
#' Each column is a sample, and each row is a gene. 
#' @param module a data.frame of two column. The first column is module name, 
#' the second column are genes in this module.
#'
#' @return a data.frame, means the mean of gene expression value in
#' the same module
#' @export
#'
#' @examples
#' data(geneExpress)
#' data(module)
#' result <- cal_mean_module(geneExpress, module)
cal_mean_module <- function(geneExpress, module) {
    genes <- rownames(geneExpress)
    output_module <- matrix(0, nrow(module), 2)
    rownames(output_module) <- module[, 1]
    for (i in seq_len(nrow(module))) {
        modulen <- unlist(strsplit(module[i, 2], ","))
        modulen <- intersect(modulen, genes)
        modulenDf <- geneExpress[modulen, ]
        output_module[i, ] <- colMeans(modulenDf)
    }
    as.data.frame(output_module)
}

#' Do difference analysis of SNP data
#'
#' @param snpDf data.frame of SNP data, each column is a sample, 
#' and each row is a SNP. 
#' @param sampleGroup vector of sample group.
#' @param combineMethod Method of combining the
#' pvalue of multiple snp in a gene.
#' @return data.frame
#' @export
#'
#' @examples
#' \donttest{
#' library(TCGAbiolinks)
#' query <- GDCquery(
#'     project = "TCGA-CHOL",
#'     data.category = "Simple Nucleotide Variation",
#'     access = "open",
#'     legacy = FALSE,
#'     data.type = "Masked Somatic Mutation",
#'     workflow.type = "Aliquot Ensemble Somatic Variant Merging and Masking"
#' )
#' GDCdownload(query)
#' data_snp <- GDCprepare(query)
#' samples <- unique(data_snp$Tumor_Sample_Barcode)
#' sampleGroup <- sample(c("A", "B"), length(samples), replace = TRUE)
#' names(sampleGroup) <- samples
#' pvalue <- differential_SNP_tcga(snpData = data_snp, 
#'     sampleGroup = sampleGroup)
#' }
#' # use demo data
#' snpDf <- matrix(sample(c("mutation", NA), 100, replace = TRUE), 10, 10)
#' snpDf <- as.data.frame(snpDf)
#' sampleGroup <- sample(c("A", "B"), 10, replace = TRUE)
#' result <- differential_SNP(snpDf, sampleGroup)
differential_SNP <- function(snpDf, sampleGroup, combineMethod = min) {
    snpDf[!is.na(snpDf)] <- "mutation"
    snpDf[is.na(snpDf)] <- "wild"
    sampleGroup <- sampleGroup[!is.na(sampleGroup)]
    type1 <- which(sampleGroup == names(table(sampleGroup))[1])
    type2 <- which(sampleGroup == names(table(sampleGroup))[2])
    pvalue <- rep(0, nrow(snpDf))
    estimate <- rep(0, nrow(snpDf))
    for (i in seq_len(nrow(snpDf))) {
        type1_freq <- table(as.character(snpDf[i, type1]))
        type2_freq <- table(as.character(snpDf[i, type2]))
        df <- data.frame(
            type1 = as.numeric(type1_freq[c("wild", "mutation")]),
            type2 = as.numeric(type2_freq[c("wild", "mutation")])
        )
        df[is.na(df)] <- 0
        fish <- stats::fisher.test(df)
        pvalue[i] <- fish$p.value
        estimate[i] <- fish$estimate
    }
    names(pvalue) <- names(estimate) <- sub("_.*", "", rownames(snpDf))
    if (!is.null(combineMethod)) {
        pvalue <- stats::aggregate(pvalue, by = list(names(pvalue)),
            FUN = combineMethod)
        estimate <- stats::aggregate(estimate,
            by = list(names(estimate)), FUN = mean)
        return(data.frame(gene = pvalue[, 1], pvalue = pvalue[, 2],
            estimate = estimate[, 2]))
    } else {
        return(data.frame(pvalue = pvalue, estimate = estimate))
    }
}

#' combine pvalues of SNP difference analysis result
#'
#' @param snpResult data.frame of SNP difference analysis result.
#' @param snp2gene data frame of two column: snp and gene.
#' @param combineMethod Method of combining the
#' pvalue of multiple snp in a gene.
#' @return data.frame
#' @export
#' @examples
#' snpResult <- data.frame(pvalue = runif(100), estimate = runif(100))
#' rownames(snpResult) <- paste0("snp", seq_len(100))
#' snp2gene <- data.frame(snp = rownames(snpResult), 
#'     gene = rep(paste0("gene", seq_len(20)), 5))
#' result <- combine_pvalue(snpResult, snp2gene)
combine_pvalue <- function(snpResult, snp2gene, combineMethod = min) {
        pvalue <- snpResult$pvalue
        estimate <- snpResult$estimate
        genes <- snp2gene[, 2]
        names(genes) <- snp2gene[, 1]
        snps <- rownames(snpResult)
        names(pvalue) <- names(estimate) <- genes[snps]
        pvalue <- stats::aggregate(pvalue, by = list(names(pvalue)),
            FUN = combineMethod)
        estimate <- stats::aggregate(estimate, by = list(names(estimate)),
            FUN = mean)
        return(data.frame(gene = pvalue[, 1], pvalue = pvalue[, 2],
            estimate = estimate[, 2]))

}

#' Do difference analysis of SNP data downloaded from TCGAbiolinks
#'
#' @param snpData data.frame of SNP data downloaded from TCGAbiolinks
#' @param sampleGroup vector of sample group
#' @param combineMethod Method of combining the pvalue of
#' multiple snp in a gene.
#' @return data.frame
#' @export
#'
#' @examples
#' \donttest{
#' library(TCGAbiolinks)
#' query <- GDCquery(
#'     project = "TCGA-CHOL",
#'     data.category = "Simple Nucleotide Variation",
#'     access = "open",
#'     legacy = FALSE,
#'     data.type = "Masked Somatic Mutation",
#'     workflow.type = "Aliquot Ensemble Somatic Variant Merging and Masking"
#' )
#' GDCdownload(query)
#' data_snp <- GDCprepare(query)
#' samples <- unique(data_snp$Tumor_Sample_Barcode)
#' sampleGroup <- sample(c("A", "B"), length(samples), replace = TRUE)
#' names(sampleGroup) <- samples
#' pvalue <- differential_SNP_tcga(snpData = data_snp, 
#'     sampleGroup = sampleGroup)
#' }
#' # use demo data
#' snpDf <- matrix(sample(c("mutation", NA), 100, replace = TRUE), 10, 10)
#' snpDf <- as.data.frame(snpDf)
#' sampleGroup <- sample(c("A", "B"), 10, replace = TRUE)
#' result <- differential_SNP(snpDf, sampleGroup)
differential_SNP_tcga <- function(snpData, sampleGroup, combineMethod = NULL) {
    Tumor_Sample_Barcode <- Variant_Classification <- NULL
    snpName <- paste(snpData$Hugo_Symbol, snpData$Start_Position, sep = "_")
    # snpData <- snpData[, c("Hugo_Symbol", "Start_Position", "Chromosome",
    #         "Variant_Classification", "Tumor_Sample_Barcode",
    #         "Variant_Type", "dbSNP_RS", "Mutation_Status",
    #         # "MAX_AF",
    #         "Reference_Allele", "Tumor_Seq_Allele1", "Tumor_Seq_Allele2",
    #         "Match_Norm_Validation_Allele1", "Match_Norm_Validation_Allele2")]
    snpData <- snpData[, c("Variant_Classification", "Tumor_Sample_Barcode")]
    snpData$snp <- snpName
    snpData <- tidyr::spread(snpData, Tumor_Sample_Barcode,
        Variant_Classification)
    snpData <- as.data.frame(snpData)
    i <- match(colnames(snpData), names(sampleGroup))
    sampleGroup <- sampleGroup[i]
    rownames(snpData) <- snpData$snp
    snpData <- snpData[, -1]
    pvalue <- differential_SNP(snpDf = snpData, sampleGroup = sampleGroup,
        combineMethod = combineMethod)
    return(pvalue)
}

#' Do difference analysis of SNP data downloaded from GEO
#'
#' @param snpData data.frame of SNP data downloaded from GEO
#' @param sampleGroup vector of sample group
#' @param method one of "Chisquare", "fisher",
#' and "CATT"(Cochran-Armitage trend test)
#' @return data.frame
#' @export
#' @examples
#' \donttest{
#' file1 <- read.table("GSE66903_series_matrix.txt.gz",
#'     fill=TRUE, comment.char="!", header = TRUE)
#' rownames(file1) <- file1[, 1]
#' snpData <- file1[, -1]
#' sampleGroup <- sample(c("A", "B"), ncol(snpData ), replace = TRUE)
#' names(sampleGroup) <- colnames(snpData)
#' snpData <- SNP_QC(snpData)
#' sampleGroup <- sample(c("A", "B"), ncol(snpData ), replace = TRUE)
#' result1 <- differential_SNP_GEO(snpData = snpData,
#'     sampleGroup = sampleGroup, method = "Chisquare")
#' }
#' # use demo data
#' snpDf <- matrix(sample(c("AA", "Aa", "aa"), 100, replace = TRUE), 10, 10)
#' snpDf <- as.data.frame(snpDf)
#' sampleGroup <- sample(c("A", "B"), 10, replace = TRUE)
#' result <- differential_SNP_GEO(snpDf, sampleGroup, method = "fisher")
differential_SNP_GEO <- function(snpData, sampleGroup, method = "Chisquare") {
    snpDf <- as.matrix(snpData)
    sampleGroup <- sampleGroup[!is.na(sampleGroup)]
    type1 <- which(sampleGroup == names(table(sampleGroup))[1])
    type2 <- which(sampleGroup == names(table(sampleGroup))[2])
    pvalue <- rep(1, nrow(snpDf))
    estimate <- rep(0, nrow(snpDf))
    for (i in seq_len(nrow(snpDf))) {
        type1_freq <- table(snpDf[i, type1])
        type2_freq <- table(snpDf[i, type2])
        types <- unique(snpDf[i, ])
        df <- data.frame(
            type1_freq = as.numeric(type1_freq[types]),
            type2_freq = as.numeric(type2_freq[types])
        )
        df[is.na(df)] <- 0
        if (nrow(df) > 2) {
            if (method == "fisher") {
                fish <- stats::fisher.test(df)
                pvalue[i] <- fish$p.value
                if (nrow(df) == 2) {
                        estimate[i] <- fish$estimate
                }
            }

            if (method == "Chisquare") {
                    pvalue[i] <- stats::chisq.test(df)$p.value
            }

            if(method == "CATT") {
                    pvalue[i] <- CATT::CATT(table = t(df))$p.value
            }
        }

    }
    names(pvalue) <- names(estimate) <- rownames(snpDf)

    return(data.frame(pvalue = pvalue, estimate = estimate))
}


get_maf <- function(x) {
    x <- x[x != "NoCall"]
    freq <- strsplit(x, split = "") |> unlist() |> table()
    min(freq) / sum(freq)
}

get_hwe <- function(x) {
    x <- x[x != "NoCall"]
    aa <- table(x)
    table_x <- as.numeric(aa)
    names(table_x) <- names(aa)
    # table_x <- table_x[sort(names(table_x))]
    freq <- strsplit(x, split = "") |> unlist() |> table()
    # freq <- freq[sort(names(freq))]
    table_y <- rep(0, 3)


    names(table_y) <- names(table_x)
    freq1 <- freq[1]/ sum(freq)
    freq2 <- freq[2]/ sum(freq)
    sum_freq <- length(x)
    table_y[paste0(names(freq)[1], names(freq)[1])] <- freq1 * freq1 * sum_freq
    if (length(table_x) > 1) {
        table_y[paste0(names(freq)[1],
            names(freq)[2])] <- freq1 * freq2 * sum_freq * 2
        table_y[paste0(names(freq)[2],
            names(freq)[1])] <- freq1 * freq2 * sum_freq * 2
    }
    if (length(table_x) > 2) {
            table_y[paste0(names(freq)[2],
                names(freq)[2])] <- freq2 * freq2 * sum_freq
    }
    df <- data.frame(table_x, table_y[names(table(x))])
    stats::chisq.test(df)$p.value
}

#' Do quality control of SNP data downloaded from TCGAbiolinks
#'
#' @param snpData data.frame of SNP data downloaded from TCGAbiolinks
#' @param geon filters out all variants with missing call rates
#' exceeding the provided value (default 0.02) to be removed
#' @param mind filters out all samples with missing call rates exceeding
#' the provided value (default 0.02) to be removed
#' @param maf filters out all variants with minor allele frequency below
#' the provided threshold
#' @param hwe filters out all variants which have Hardy-Weinberg
#' equilibrium exact test p-value below the provided threshold
#' @param miss character of miss value
#' @return data.frame
#' @export
#' @examples
#' # use demo data
#' snpDf <- matrix(sample(c("AA", "Aa", "aa"), 100, replace = TRUE), 10, 10)
#' snpDf <- as.data.frame(snpDf)
#' sampleGroup <- sample(c("A", "B"), 10, replace = TRUE)
#' result <- SNP_QC(snpDf)
SNP_QC <- function(snpData, geon = 0.02, mind = 0.02, maf = 0.05,
                    hwe = 1e-6, miss = "NoCall") {
    snpData_mat <- as.matrix(snpData)
    ## filter by 0.2
    aa <- snpData_mat |> apply(MARGIN = 2, FUN = function(x) {
            table(x)[miss] / length(x)
        }
    )

    aa[is.na(aa)] <- 0
    snpData_mat <- snpData_mat[, aa < 0.2]

    bb <- snpData_mat |> apply(MARGIN = 1, FUN = function(x) {
        table(x)[miss] / length(x)
        }
    )

    bb[is.na(bb)] <- 0
    snpData_mat <- snpData_mat[bb < 0.2, ]

    ## filter by cutoff
    aa <- snpData_mat |> apply(MARGIN = 2, FUN = function(x)
        {table(x)[miss] / length(x)})

    aa[is.na(aa)] <- 0
    snpData_mat <- snpData_mat[, aa < mind]

    bb <- snpData_mat |> apply(MARGIN = 1, FUN = function(x)
        {table(x)[miss] / length(x)})

    bb[is.na(bb)] <- 0
    snpData_mat <- snpData_mat[bb < geon, ]

    ## maf
    MAF <- snpData_mat |> apply(MARGIN = 1, FUN = get_maf)
    snpData_mat <- snpData_mat[MAF > maf, ]
    HWE <- snpData_mat |> apply(MARGIN = 1, FUN = get_hwe)
    snpData_mat <- snpData_mat[HWE > hwe, ] |> as.data.frame()
    return(snpData_mat)
}

#' Do difference analysis of RNA-seq data
#'
#' @title differential_RNA
#' @rdname differential_RNA
#' @param counts a dataframe or numeric matrix of raw counts data, 
#' or SummarizedExperiment object
#' @param group sample groups
#' @param groupCol group column
#' @param method one of "DESeq2", "edgeR" , "limma", "dearseq",
#' "NOISeq", "Wilcoxon", and "auto".
#' @param geneLength a vector of gene length.
#' @param gccontent a vector of gene GC content.
#' @param filter if TRUE, use filterByExpr to filter genes.
#' @param edgeRNorm if TRUE, use edgeR to do normalization for dearseq method.
#' @param adjust.method character string specifying the method used to
#' adjust p-values for multiple testing.
#' See \link{p.adjust} for possible values.
#' @param useTopconfects if TRUE, use topconfects to provide a
#'    more biologically useful ranked gene list.
#' @param ucscData Logical, whether the data comes from UCSC Xena.
#' @importFrom plyr rename
#' @importFrom SummarizedExperiment assays
#' @importFrom SummarizedExperiment colData
#' @importFrom utils methods
#' @import methods
#' @import cqn
#' @return data.frame
#' @export
#'
#' @examples
#' \donttest{
# use `TCGAbiolinks` to download TCGA data
#' library(TCGAbiolinks)
#'
#' query <- GDCquery(
#'     project = "TCGA-ACC",
#'     data.category = "Transcriptome Profiling",
#'     data.type = "Gene Expression Quantification",
#'     workflow.type = "STAR - Counts"
#' )
#'
#' GDCdownload(query,
#'     method = "api", files.per.chunk = 3,
#'     directory = Your_Path
#' )
#'
#' dataRNA <- GDCprepare(
#'     query = query, directory = Your_Path,
#'     save = TRUE, save.filename = "dataRNA.RData"
#' )
#' ## get raw count matrix
#' dataPrep <- TCGAanalyze_Preprocessing(
#'     object = dataRNA,
#'     cor.cut = 0.6,
#'     datatype = "STAR - Counts"
#' )
#'
#' # Use `differential_RNA` to do difference analysis.
#' # We provide the data of human gene length and GC content in `gene_cov`.
#' group <- sample(c("grp1", "grp2"), ncol(dataPrep), replace = TRUE)
#' library(cqn) # To avoid reporting errors: there is no function "rq"
#' ## get gene length and GC content
#' library(org.Hs.eg.db)
#' genes_bitr <- bitr(rownames(gene_cov),
#'     fromType = "ENTREZID", toType = "ENSEMBL",
#'     OrgDb = org.Hs.eg.db, drop = TRUE
#' )
#' genes_bitr <- genes_bitr[!duplicated(genes_bitr[, 2]), ]
#' gene_cov2 <- gene_cov[genes_bitr$ENTREZID, ]
#' rownames(gene_cov2) <- genes_bitr$ENSEMBL
#' genes <- intersect(rownames(dataPrep), rownames(gene_cov2))
#' dataPrep <- dataPrep[genes, ]
#' geneLength <- gene_cov2(genes, "length")
#' gccontent <- gene_cov2(genes, "GC")
#' names(geneLength) <- names(gccontent) <- genes
#' ##    Difference analysis
#' DEGAll <- differential_RNA(
#'     counts = dataPrep, group = group,
#'     geneLength = geneLength, gccontent = gccontent
#' )
#' # Use `clusterProfiler` to do enrichment analytics:
#' diffGenes <- DEGAll$logFC
#' names(diffGenes) <- rownames(DEGAll)
#' diffGenes <- sort(diffGenes, decreasing = TRUE)
#' library(clusterProfiler)
#' library(enrichplot)
#' library(org.Hs.eg.db)
#' gsego <- gseGO(gene = diffGenes, OrgDb = org.Hs.eg.db, keyType = "ENSEMBL")
#' dotplot(gsego)
#' }
#' # use user-defined data
#' df <- matrix(rnbinom(400, mu = 4, size = 10), 25, 16)
#' df <- as.data.frame(df)
#' rownames(df) <- paste0("gene", 1:25)
#' colnames(df) <- paste0("sample", 1:16)
#' group <- sample(c("group1", "group2"), 16, replace = TRUE)
#' result <- differential_RNA(counts = df, group = group,
#'     filte = FALSE, method = "Wilcoxon")
#' # use SummarizedExperiment object input
#' df <- matrix(rnbinom(400, mu = 4, size = 10), 25, 16)
#' rownames(df) <- paste0("gene", 1:25)
#' colnames(df) <- paste0("sample", 1:16)
#' group <- sample(c("group1", "group2"), 16, replace = TRUE)
#' 
#' nrows <- 200; ncols <- 20
#'  counts <- matrix(
#'    runif(nrows * ncols, 1, 1e4), nrows,
#'    dimnames = list(paste0("cg",1:200),paste0("S",1:20))
#' )
#' 
#' colData <- S4Vectors::DataFrame(
#'   row.names = paste0("sample", 1:16),
#'   group = group
#' )
#' data <- SummarizedExperiment::SummarizedExperiment(
#'          assays=S4Vectors::SimpleList(counts=df),
#'          colData = colData)
#' 
#' result <- differential_RNA(counts = data, groupCol = "group",
#'     filte = FALSE, method = "Wilcoxon") 
#' @importFrom plyr rename
#' @import cqn
differential_RNA <- function(counts, group, groupCol, method = "limma", 
                    geneLength = NULL,
                    gccontent = NULL, filter = TRUE, edgeRNorm = TRUE,
                    adjust.method = "BH", useTopconfects = TRUE, 
                    ucscData = FALSE) {
    method <- match.arg(method, c("DESeq2", "edgeR", "limma",
        "dearseq", "Wilcoxon", "NOISeq", "auto"))

    if (ucscData) {
        class(counts) <- "data.frame"
        counts[, 1] <- gsub("\\..*", "", counts[, 1])
        rownames(counts) <- counts[, 1]
        counts <- counts[, -1]
        counts <- round(2^counts) - 1
    }


    if (inherits(counts,  "SummarizedExperiment")) {
        se <- counts
        counts <- assays(se)$counts
        group <- colData(se)[, groupCol]
        names(group) <- rownames(colData(se))
    }


    cols <- !duplicated(colnames(counts))
    counts <- counts[, cols]
    group <- group[cols]
    if (min(table(group)) > 4 && method == "auto") {
        method <- "Wilcoxon"
    } else {
        method <- "limma"
    }

    ## use cqn to correct the bias
    correct<- TRUE
    uCovar <- NULL
    if (is.null(geneLength) || is.null(gccontent)) {
        correct<- FALSE
    } else {
        genes_gc <- intersect(names(geneLength), names(gccontent))
        uCovar <- data.frame(length = geneLength[genes_gc],
            gccontent = gccontent[genes_gc])
        rownames(uCovar) <- genes_gc
        counts <- counts[genes_gc, ]
    }
    d.mont <- edgeR::DGEList(counts = counts, group = group, genes = uCovar)
    if (filter) {
        keep <- edgeR::filterByExpr(d.mont)
        d.mont <- d.mont[keep, keep.lib.sizes = FALSE]
        counts <- counts[keep, ]
    }
    if (correct) {
        geneLength <- geneLength[rownames(counts)]
        gccontent <- gccontent[rownames(counts)]
        cqn.subset <- cqn::cqn(counts, lengths = geneLength, x = gccontent)
    }


    if (method == "DESeq2") {
        coldata <- data.frame(group)

        dds <- DESeq2::DESeqDataSetFromMatrix(
            countData = counts,
            colData = coldata, design = ~group
        )
        

        if (correct) {
            cqnOffset <- cqn.subset$glm.offset
            cqnNormFactors <- exp(cqnOffset)
            ## divide out the geometric mean
            ## https://support.bioconductor.org/p/89239/
            ## https://support.bioconductor.org/p/95683/
            normFactors <- cqnNormFactors / exp(rowMeans(log(cqnNormFactors)))
            DESeq2::normalizationFactors(dds) <- normFactors
        }
        DEGAll <- DESeq2::DESeq(dds)
        DEGAll_table <- NULL
        if (useTopconfects) {
            DEGAll_table <-
                topconfects::deseq2_confects(DEGAll, step = 0.05)$table
            rownames(DEGAll_table) <- DEGAll_table$name
        }
        DEGAll <- DEGAll |>
            DESeq2::results(pAdjustMethod = adjust.method) |>
            as.data.frame() |>
            rename(c("log2FoldChange" = "logFC")) |>
            rename(c("pvalue" = "P.Value")) |>
            rename(c("padj" = "adj.P.Val"))
        DEGAll$length <- geneLength[rownames(DEGAll)]
        DEGAll$gccontent <- gccontent[rownames(DEGAll)]
        # DEGAll <- DEGAll[, c(ncol(DEGAll) - 1,
        #     ncol(DEGAll), 1:(ncol(DEGAll) - 2))]
        DEGAll <- DEGAll[, c(ncol(DEGAll) - 1, ncol(DEGAll),
            seq_len(ncol(DEGAll) - 2))]
        if (!is.null(DEGAll_table)) {
            genes <- intersect(rownames(DEGAll), rownames(DEGAll_table))
            DEGAll <- cbind(DEGAll[genes, ], DEGAll_table[genes, ])
            DEGAll <- DEGAll[order(DEGAll$P.Value), ]
        }
    } else {
        if (correct) {
            ## with cqn, there is no need to normalize
            ## using the normalization tools
            ## from edgeR, such as calcNormFactors.
            d.mont$offset <- cqn.subset$glm.offset
        } else {
            ## TMM Normalization
            d.mont <- edgeR::calcNormFactors(d.mont, method = "TMM")
        }
        if (method == "edgeR") {
            # design <- stats::model.matrix(~ group)
            design <- stats::model.matrix(~ d.mont$sample$group)
            if (min(table(d.mont$sample$group)) > 1) {
                d.mont <- edgeR::estimateDisp(d.mont, design) |>
                    edgeR::estimateGLMCommonDisp(design = design)
                DEGAll <- edgeR::glmQLFit(d.mont, design = design)
                DEGAll_table <- NULL
                if (useTopconfects) {
                    DEGAll_table <- topconfects::edger_confects(DEGAll,
                        fdr = 0.05,
                        coef = ncol(DEGAll$design),
                        step = 0.05
                    )$table
                    rownames(DEGAll_table) <- DEGAll_table$name
                }
                # edgeR::topTags(n = nrow(d.mont$counts)) |>
                DEGAll <- DEGAll |>
                    edgeR::glmQLFTest(coef = ncol(DEGAll$design)) |>
                    edgeR::topTags(n = Inf, adjust.method = adjust.method) |>
                    as.data.frame() |>
                    rename(c("FDR" = "adj.P.Val")) |>
                    rename(c("PValue" = "P.Value"))
                if (!is.null(DEGAll_table)) {
                    genes <- intersect(rownames(DEGAll), rownames(DEGAll_table))
                    DEGAll <- cbind(DEGAll[genes, ], DEGAll_table[genes, ])
                    DEGAll <- DEGAll[order(DEGAll$P.Value), ]
                }
            } else {
                DEGAll <- edgeR::glmFit(d.mont, dispersion = 0)
                DEGAll <- DEGAll |>
                    edgeR::glmLRT(coef = ncol(DEGAll$design)) |>
                    edgeR::topTags(n = Inf, adjust.method = adjust.method) |>
                    as.data.frame() |>
                    rename(c("FDR" = "adj.P.Val")) |>
                    rename(c("PValue" = "P.Value"))
            }
        }

        if (method == "limma") {
            comparison <- paste(unique(group), collapse = "-")
            group <- factor(group)
            design <- stats::model.matrix(~ 0 + group)
            colnames(design) <- levels(group)
            contrast.matrix <- limma::makeContrasts(
                contrasts = comparison,
                levels = design
            )
            DEGAll <- limma::voom(d.mont, design = design, plot = FALSE) |>
                limma::lmFit(design)
            DEGAll_table <- NULL
            if (useTopconfects) {
                DEGAll_table <- topconfects::limma_confects(DEGAll,
                    coef = 1,
                    fdr = 0.05
                )$table
                rownames(DEGAll_table) <- DEGAll_table$name
            }
            DEGAll <- DEGAll |>
                limma::contrasts.fit(contrast.matrix) |>
                limma::eBayes() |>
                limma::topTable(number = Inf, adjust.method = adjust.method)
            if (!is.null(DEGAll_table)) {
                genes <- intersect(rownames(DEGAll), rownames(DEGAll_table))
                DEGAll <- cbind(DEGAll[genes, ], DEGAll_table[genes, ])
                DEGAll <- DEGAll[order(DEGAll$P.Value), ]
            }
        }

        if (method == "dearseq") {
            group[group == unique(group)[1]] <- 1
            group[group == unique(group)[2]] <- 2
            conditions <- matrix(as.numeric(group), ncol = 1)
            dearseqTest <- "asymptotic"
            if (edgeRNorm) {
                count_norm <- edgeR::cpm(d.mont, log = TRUE)
                DEGAll <- dearseq::dear_seq(
                    exprmat = count_norm, variables2test = conditions,
                    which_test = dearseqTest, parallel_comp = FALSE,
                    preprocessed = TRUE
                )
            } else {
                DEGAll <- dearseq::dear_seq(
                    exprmat = as.matrix(counts), variables2test = conditions,
                    which_test = dearseqTest, parallel_comp = FALSE,
                    preprocessed = FALSE,
                    padjust_methods = adjust.method
                )
            }
            DEGAll <- DEGAll$pvals |>
                rename(c("adjPval" = "adj.P.Val")) |>
                rename(c("rawPval" = "P.Value"))
        }

        if (method == "Wilcoxon") {
            count_norm <- edgeR::cpm(d.mont, log = TRUE) |> as.data.frame()
            pvalues <- rep(0, nrow(count_norm))

            count_disease <- as.matrix(count_norm[, group == unique(group)[1]])
            count_normal <- as.matrix(count_norm[, group == unique(group)[2]])
            for (i in seq_len(length(pvalues))) {
                pvalues[i] <- stats::wilcox.test(count_disease[i, ],
                    count_normal[i, ])$p.value
            }
            fdr <- stats::p.adjust(pvalues, method = adjust.method)
            DEGAll <- data.frame(P.Value = pvalues, adj.P.Val = fdr)
            rownames(DEGAll) <- rownames(count_norm)
        }

        if (method == "NOISeq") {
            conditions <- factor(group)
            data <- NOISeq::readData(data = counts,
                factors = as.data.frame(conditions))
            res <- NOISeq::noiseqbio(data,
                k = 0.5, norm = "tmm", factor = "conditions",
                random.seed = 12345, filter = 1, cv.cutoff = 100, cpm = 1
            )
            DEGAll <- NOISeq::degenes(res, q = 0, M = NULL) |>
                rename(c("prob" = "P.Value"))
            DEGAll$adj.P.Val <- DEGAll$P.Value
        }
    }
    if ("P.Value" %in% colnames(DEGAll)) {
        DEGAll <- DEGAll[!is.na(DEGAll[, "P.Value"]), ]
    }

    return(DEGAll)
}

test_that("can parse example diff_RNA_ucsc", {
    skip_on_cran()
    df <- matrix(rnbinom(400, mu = 4, size = 10), 25, 16)
    df <- as.data.frame(df)
    rownames(df) <- paste0("gene", 1:25)
    colnames(df) <- paste0("sample", 1:16)
    df <- log2(df + 1)
    group <- sample(c("group1", "group2"), 16, replace = TRUE)
    df <- cbind(rownames(df), df)
    result <- differential_RNA(counts = df, group = group, 
        filte = FALSE, method = "limma", ucscData = TRUE)
    expect_true( "P.Value"    %in% colnames(result))
})

#' Do difference analysis of gene level copy number variation data
#'
#' @param cnvData data.frame of CNV data, each column is a sample, 
#' and each row is a CNV. 
#' @param sampleGroup vector of sample group
#' @param method method to do diffenenital analysis, 
#' one of "Chisquare", "fisher",
#' and "CATT"(Cochran-Armitage trend test)
#' @param adjust.method adjust.method, one of "holm", "hochberg", "hommel", 
#' "bonferroni", "BH", "BY", "fdr", and "none". 
#' @param ... parameters for "Chisquare", "fisher",
#' and "CATT"(Cochran-Armitage trend test)
#' @return data.frame with pvalue and estimate
#' @export
#' 
#' @examples
#' \donttest{
#' # use TCGAbiolinks data as example
#' library(TCGAbiolinks)
#' query <- GDCquery(
#'         project = "TCGA-ACC",
#'         data.category = "Copy Number Variation",
#'         data.type = "Gene Level Copy Number",
#'         access = "open"
#' )
#' GDCdownload(query)
#' cnvData <- GDCprepare(query)
#' aa <- assays(cnvData)$copy_number
#' bb <- aa
#' aa[bb == 2] <- 0
#' aa[bb < 2] <- -1
#' aa[bb > 2] <- 1
#' sampleGroup <- sample(c("A", "B"), ncol(cnvData), replace = TRUE)
#' diffCnv <- differential_CNV(aa, sampleGroup)
#'
#' # Use sangerbox CNV data as example
#' cnvData <- fread("Merge_GeneLevelCopyNumber.txt")
#' class(cnvData) <- "data.frame"
#' rownames(cnvData) <- cnvData[, 1]
#' cnvData <- cnvData[, -c(1, 2, 3)]
#' sampleGroup <- sample(c("A", "B"), ncol(cnvData), replace = TRUE)
#' diffCnv <- differential_CNV(cnvData, sampleGroup)
#' }
#' # use random data as example
#' aa <- matrix(sample(c(0, 1, -1), 200, replace = TRUE), 25, 8)
#' rownames(aa) <- paste0("gene", 1:25)
#' colnames(aa) <- paste0("sample", 1:8)
#' sampleGroup <- sample(c("A", "B"), ncol(aa), replace = TRUE)
#' diffCnv <- differential_CNV(aa, sampleGroup)
differential_CNV <- function(cnvData, sampleGroup,
                    method = "Chisquare",
                    adjust.method = "BH", ...) {
    type1 <- which(sampleGroup == unique(sampleGroup)[1])
    type2 <- which(sampleGroup == unique(sampleGroup)[2])
    pvalue <- rep(1, nrow(cnvData))
    estimate <- rep(0, nrow(cnvData))
    for (i in seq_len(nrow(cnvData))) {
        type1_freq <- table(as.character(cnvData[i, type1]))
        type2_freq <- table(as.character(cnvData[i, type2]))
        df <- data.frame(
            type1 = as.numeric(type1_freq[c("-1", "0", "1")]),
            type2 = as.numeric(type2_freq[c("-1", "0", "1")])
        )
        df[is.na(df)] <- 0
        df <- df[rowSums(df) > 0, ]
        if (nrow(df) > 2) {
            if (method == "fisher") {
                fish <- stats::fisher.test(df, ...)
                pvalue[i] <- fish$p.value
                if (nrow(df) == 2) {
                        estimate[i] <- fish$estimate
                }
            }

            if (method == "Chisquare") {
                    pvalue[i] <- stats::chisq.test(df, ...)$p.value
            }

            if(method == "CATT") {
                    pvalue[i] <- CATT::CATT(table = t(df), ...)$p.value
            }
        }
    }

    adj.P.Val <- stats::p.adjust(pvalue, method = adjust.method)
    gene <- gsub("\\..*", "", rownames(cnvData))
    result <- data.frame(gene = gene, P.Value = pvalue,
        adj.P.Val = adj.P.Val, estimate = estimate)
    rownames(result) <- gene
    result
}

test_that("can parse example diff_CNV", {
    aa <- matrix(sample(c(0, 1, -1), 200, replace = TRUE), 25, 8)
    rownames(aa) <- paste0("gene", 1:25)
    colnames(aa) <- paste0("sample", 1:8)
    sampleGroup <- sample(c("A", "B"), ncol(aa), replace = TRUE)
    diffCnv <- differential_CNV(aa, sampleGroup)
    expect_true( "P.Value"  %in% colnames(diffCnv))
})

#' Convert ENSEMBL gene id to gene Symbol in TCGA
#'
#' @param profiles a data.frame of gene expression data, 
#' each column is a sample, 
#' and each row is a gene. 
#' @param toType one of 'keytypes(org.Hs.eg.db)'
#'
#' @return a data.frame, gene symbols and their expression value
#' @export
#'
#' @examples
#' library(org.Hs.eg.db)
#' data(profile)
#' result <- id_conversion_TCGA(profile)
id_conversion_TCGA <- function(profiles, toType = "SYMBOL") {
    rownames(profiles) <- gsub("\\..*", "", rownames(profiles))
    genes <- clusterProfiler::bitr(rownames(profiles),
        fromType = "ENSEMBL",
        toType = toType, OrgDb = org.Hs.eg.db::org.Hs.eg.db, drop = FALSE
    )

    genes <- genes[!duplicated(genes[, 1]), ]
    rownames(genes) <- genes[, 1]
    profiles2 <- as.matrix(profiles)
    rownames(profiles2) <- genes[rownames(profiles), 2]
    profiles2 <- profiles2[!is.na(rownames(profiles2)), ]
    return(profiles2)
}

#' Preparer file for chi-square test
#'
#' @param cnv result of ann_merge()
#'
#' @return a matrix
#' @export
#'
#' @examples
#' cnv <- matrix(c(
#'     -1.09150, -1.47120, -0.87050, -0.50880,
#'     -0.50880, 2.0, 2.0, 2.0, 2.0, 2.0, 2.601962, 2.621332, 2.621332,
#'     2.621332, 2.621332, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
#'     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0
#' ), nrow = 5)
#' cnv <- as.data.frame(cnv)
#' rownames(cnv) <- c("AJAP1", "FHAD1", "CLCNKB", "CROCCP2", "AL137798.3")
#' colnames(cnv) <- c(
#'     "TCGA-DD-A4NS-10A-01D-A30U-01", "TCGA-ED-A82E-01A-11D-A34Y-01",
#'     "TCGA-WQ-A9G7-01A-11D-A36W-01", "TCGA-DD-AADN-01A-11D-A40Q-01",
#'     "TCGA-ZS-A9CD-10A-01D-A36Z-01", "TCGA-DD-A1EB-11A-11D-A12Y-01"
#' )
#' cnv_chi_file <- prepare_chi(cnv)
prepare_chi <- function(cnv) {
    file1 <- cnv
    sampless <- rep(0, ncol(file1))
    for (i in seq_len(length(sampless)))
    {
        a <- unlist(strsplit(colnames(file1)[i], "-"))[4]
        sampless[i] <- substring(a, 1, 1)
    }

    cnv_chi <- matrix(0, nrow(cnv), 4)
    rownames(cnv_chi) <- rownames(cnv)
    colnames(cnv_chi) <- c("normalCNV", "normalWild", "tumorCNV", "tumorWild")
    for (i in seq_len(nrow(file1)))
    {
        normalCNV <- 0
        normalWild <- 0
        tumorCNV <- 0
        tumorWild <- 0
        for (j in seq_len(ncol(file1)))
        {
            if ((sampless[j] == "1") &&
                (abs(as.numeric(file1[i, j]) - 2) > 0.5)) {
                normalCNV <- normalCNV + 1
            }
            if ((sampless[j] == "1") &&
                (abs(as.numeric(file1[i, j]) - 2) <= 0.5)) {
                normalWild <- normalWild + 1
            }
            if ((sampless[j] == "0") &&
                (abs(as.numeric(file1[i, j]) - 2) > 0.5)) {
                tumorCNV <- tumorCNV + 1
            }
            if ((sampless[j] == "0") &&
                (abs(as.numeric(file1[i, j]) - 2) <= 0.5)) {
                tumorWild <- tumorWild + 1
            }
        }
        cnv_chi[i, ] <- c(normalCNV, normalWild, tumorCNV, tumorWild)
    }
    return(cnv_chi)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(GeoTcgaData)

## ----message=FALSE, warning=FALSE---------------------------------------------
# use user-defined data
df <- matrix(rnbinom(400, mu = 4, size = 10), 25, 16)
df <- as.data.frame(df)
rownames(df) <- paste0("gene", 1:25)
colnames(df) <- paste0("sample", 1:16)
group <- sample(c("group1", "group2"), 16, replace = TRUE)
result <- differential_RNA(counts = df, group = group,
    filte = FALSE, method = "Wilcoxon")
# use SummarizedExperiment object input
df <- matrix(rnbinom(400, mu = 4, size = 10), 25, 16)
rownames(df) <- paste0("gene", 1:25)
colnames(df) <- paste0("sample", 1:16)
group <- sample(c("group1", "group2"), 16, replace = TRUE)

nrows <- 200; ncols <- 20
counts <- matrix(
    runif(nrows * ncols, 1, 1e4), nrows,
    dimnames = list(paste0("cg",1:200),paste0("S",1:20))
)

colData <- S4Vectors::DataFrame(
  row.names = paste0("sample", 1:16),
  group = group
)
data <- SummarizedExperiment::SummarizedExperiment(
         assays=S4Vectors::SimpleList(counts=df),
         colData = colData)

result <- differential_RNA(counts = data, groupCol = "group",
    filte = FALSE, method = "Wilcoxon") 

## ----message=FALSE, warning=FALSE---------------------------------------------
# use user defined data
library(ChAMP)
cpgData <- matrix(runif(2000), nrow = 200, ncol = 10)
rownames(cpgData) <- paste0("cpg", seq_len(200))
colnames(cpgData) <- paste0("sample", seq_len(10))
sampleGroup <- c(rep("group1", 5), rep("group2", 5))
names(sampleGroup) <- colnames(cpgData)
cpg2gene <- data.frame(cpg = rownames(cpgData), 
    gene = rep(paste0("gene", seq_len(20)), 10))
result <- differential_methy(cpgData, sampleGroup, 
    cpg2gene = cpg2gene, normMethod = NULL)
# use SummarizedExperiment object input
library(ChAMP)
cpgData <- matrix(runif(2000), nrow = 200, ncol = 10)
rownames(cpgData) <- paste0("cpg", seq_len(200))
colnames(cpgData) <- paste0("sample", seq_len(10))
sampleGroup <- c(rep("group1", 5), rep("group2", 5))
names(sampleGroup) <- colnames(cpgData)
cpg2gene <- data.frame(cpg = rownames(cpgData), 
    gene = rep(paste0("gene", seq_len(20)), 10))
colData <- S4Vectors::DataFrame(
    row.names = colnames(cpgData),
    group = sampleGroup
)
data <- SummarizedExperiment::SummarizedExperiment(
         assays=S4Vectors::SimpleList(counts=cpgData),
         colData = colData)
result <- differential_methy(cpgData = data, 
    groupCol = "group", normMethod = NULL, 
    cpg2gene = cpg2gene)  

## ----message=FALSE, warning=FALSE---------------------------------------------
# use random data as example
aa <- matrix(sample(c(0, 1, -1), 200, replace = TRUE), 25, 8)
rownames(aa) <- paste0("gene", 1:25)
colnames(aa) <- paste0("sample", 1:8)
sampleGroup <- sample(c("A", "B"), ncol(aa), replace = TRUE)
diffCnv <- differential_CNV(aa, sampleGroup)

## ----message=FALSE, warning=FALSE---------------------------------------------
snpDf <- matrix(sample(c("AA", "Aa", "aa"), 100, replace = TRUE), 10, 10)
snpDf <- as.data.frame(snpDf)
sampleGroup <- sample(c("A", "B"), 10, replace = TRUE)
result <- SNP_QC(snpDf)

## ----message=FALSE, warning=FALSE---------------------------------------------
#' snpDf <- matrix(sample(c("mutation", NA), 100, replace = TRUE), 10, 10)
#' snpDf <- as.data.frame(snpDf)
#' sampleGroup <- sample(c("A", "B"), 10, replace = TRUE)
#' result <- differential_SNP(snpDf, sampleGroup)

## ----message=FALSE, warning=FALSE---------------------------------------------
aa <- c("MARCH1","MARC1","MARCH1","MARCH1","MARCH1")
bb <- c(2.969058399,4.722410064,8.165514853,8.24243893,8.60815086)
cc <- c(3.969058399,5.722410064,7.165514853,6.24243893,7.60815086)
file_gene_ave <- data.frame(aa=aa,bb=bb,cc=cc)
colnames(file_gene_ave) <- c("Gene", "GSM1629982", "GSM1629983")
result <- gene_ave(file_gene_ave, 1)

## -----------------------------------------------------------------------------
aa <- c("MARCH1 /// MMA","MARC1","MARCH2 /// MARCH3",
        "MARCH3 /// MARCH4","MARCH1")
bb <- c("2.969058399","4.722410064","8.165514853","8.24243893","8.60815086")
cc <- c("3.969058399","5.722410064","7.165514853","6.24243893","7.60815086")
input_file <- data.frame(aa=aa,bb=bb,cc=cc)
repAssign_result <- repAssign(input_file," /// ")
repRemove_result <- repRemove(input_file," /// ")

## ----message=FALSE, warning=FALSE---------------------------------------------
data(profile)
result <- id_conversion_TCGA(profile)

## -----------------------------------------------------------------------------
data(gene_cov)
lung_squ_count2 <- matrix(c(1,2,3,4,5,6,7,8,9),ncol=3)
rownames(lung_squ_count2) <- c("DISC1","TCOF1","SPPL3")
colnames(lung_squ_count2) <- c("sample1","sample2","sample3")
result <- countToFpkm(lung_squ_count2, keyType = "SYMBOL", 
    gene_cov = gene_cov)

## ----message=FALSE, warning=FALSE---------------------------------------------
data(gene_cov)
lung_squ_count2 <- matrix(c(0.11,0.22,0.43,0.14,0.875,
    0.66,0.77,0.18,0.29),ncol=3)
rownames(lung_squ_count2) <- c("DISC1","TCOF1","SPPL3")
colnames(lung_squ_count2) <- c("sample1","sample2","sample3")
result <- countToTpm(lung_squ_count2, keyType = "SYMBOL", 
    gene_cov = gene_cov)

## -----------------------------------------------------------------------------
sessionInfo()

#' Average the values of same genes in gene expression profile
#'
#' @param file_gene_ave a data.frame of gene expression data, 
#' each column is a sample, and each row is a gene. 
#' @param k a number, indicates which is the gene column.
#'
#' @return a data.frame, the values of same genes in gene expression profile
#' @export
#'
#' @examples
#' aa <- c("MARCH1", "MARC1", "MARCH1", "MARCH1", "MARCH1")
#' bb <- c(2.969058399, 4.722410064, 8.165514853, 8.24243893, 8.60815086)
#' cc <- c(3.969058399, 5.722410064, 7.165514853, 6.24243893, 7.60815086)
#' file_gene_ave <- data.frame(aa = aa, bb = bb, cc = cc)
#' colnames(file_gene_ave) <- c("Gene", "GSM1629982", "GSM1629983")
#'
#' result <- gene_ave(file_gene_ave, 1)
gene_ave <- function(file_gene_ave, k = 1) {
    x <- file_gene_ave[, -k]
    file_gene_ave <- as.matrix(file_gene_ave)
    rownames(file_gene_ave) <- file_gene_ave[, k]
    # x <- file_gene_ave
    ID <- rownames(file_gene_ave)
    ID <- factor(ID, levels = unique(ID))

    y <- rowsum(x, ID, reorder = FALSE, na.rm = TRUE)
    n <- rowsum(1L - is.na(x), ID, reorder = FALSE)
    return(y / n)
}

#' Handle the case where one id corresponds to multiple genes
#'
#' @param input_file input file, a data.frame or a matrix, 
#' the first column should be genes.
#' @param string a string, sep of the gene
#'
#' @return a data.frame, when an id corresponds to multiple genes,
#' the expression value is assigned to each gene
#' @export
#'
#' @examples
#' aa <- c("MARCH1 /// MMA", "MARC1", "MARCH2 /// MARCH3", 
#'     "MARCH3 /// MARCH4", "MARCH1")
#' bb <- c("2.969058399", "4.722410064", "8.165514853",
#'     "8.24243893", "8.60815086")
#' cc <- c("3.969058399", "5.722410064", "7.165514853", 
#'     "6.24243893", "7.60815086")
#' input_file <- data.frame(aa = aa, bb = bb, cc = cc)
#'
#' repAssign_result <- repAssign(input_file, " /// ")
#'
repAssign <- function(input_file, string) {
    name <- colnames(input_file)[1]
    genelist <- strsplit(input_file[, 1], string)
    geneLength <- unlist(lapply(genelist, length))
    input_file <- input_file[, -1]
    output <- apply(input_file, 2, rep, times = geneLength)
    output2 <- matrix(as.numeric(output), nrow = nrow(output))
    colnames(output2) <- colnames(output)
    output2 <- data.frame(unlist(genelist), output2, check.names = FALSE)
    colnames(output2)[1] <- name
    output2
}

#' Handle the case where one id corresponds to multiple genes
#'
#' @param input_file input file, a data.frame or a matrix, 
#' the first column should be genes.
#' @param string a string,sep of the gene
#'
#' @return a data.frame, when an id corresponds to multiple genes,
#' the expression value is deleted
#' @export
#'
#' @examples
#' aa <- c("MARCH1 /// MMA", "MARC1", "MARCH2 /// MARCH3", 
#'     "MARCH3 /// MARCH4", "MARCH1")
#' bb <- c("2.969058399", "4.722410064", "8.165514853", 
#'     "8.24243893", "8.60815086")
#' cc <- c("3.969058399", "5.722410064", "7.165514853", 
#'     "6.24243893", "7.60815086")
#' input_file <- data.frame(aa = aa, bb = bb, cc = cc)
#' repRemove_result <- repRemove(input_file, " /// ")
repRemove <- function(input_file, string) {
    unKeep <- grep(string, input_file[, 1])
    if (length(unKeep) > 0) input_file <- input_file[-unKeep, ]
    input_file
}

#' Differential analysis of Microarray data
#'
#' @param df data.frame of the omic data, each column is a sample, 
#' and each row is a gene. 
#' @param group a vector, group of samples.
#' @param method method to do differential analysis, 
#' one of "limma", "ttest", "wilcox".
#' @param adjust.method adjust.method, one of "holm", "hochberg", "hommel", 
#' "bonferroni", "BH", "BY", "fdr", and "none". 
#' @return data.frame
#' @export
#'
#' @examples
#' \donttest{
#' library(GeoTcgaData)
#' library(data.table)
#' # Use real GEO data as example
#' arrayData <- read.table("GSE54807_series_matrix.txt.gz",
#'     sep = "\t", header = TRUE,
#'         fill=TRUE, comment.char = "!", check.names=FALSE)
#' gpl <- fread("GPL6244-17930.txt", sep = "\t", header = TRUE)
#' gpl <- gpl[, c("ID", "gene_assignment")]
#' class(gpl) <- "data.frame"
#'
#' for (i in seq_len(nrow(gpl))) {
#'         aa <- strsplit(gpl[i, 2], " // ")[[1]][5]
#'         gpl[i, 2] <- as.character(strsplit(aa, " /// ")[[1]][1])
#' }
#' gpl[,1] <- as.character(gpl[,1])
#' arrayData[, 1] <- as.character(arrayData[, 1])
#' rownames(gpl) <- gpl[, 1]
#' arrayData[, 1] <- gpl[arrayData[, 1], 2]
#'
#'
#' arrayData <- repRemove(arrayData," /// ")
#'
#' # Remove rows that do not correspond to genes
#' arrayData <- arrayData[!is.na(arrayData[, 1]), ]
#' arrayData <- arrayData[!arrayData[, 1] == "", ]
#' arrayData <- arrayData[!arrayData[, 1] == "---", ]
#'
#'
#' arrayData <- arrayData[order(arrayData[, 1]), ]
#' arrayData <- gene_ave(arrayData, 1)
#'
#' keep <- apply(arrayData, 1, function(x) sum(x < 1) < (length(x)/2))
#' arrayData <- arrayData[keep, ]
#'
#' group <- c(rep("group1", 12), rep("group2", 12))
#' result <- differential_array(df = arrayData, group = group)
#' }
#' # Use random data as example
#' arrayData <- matrix(runif(200), 25, 8)
#' rownames(arrayData) <- paste0("gene", 1:25)
#' colnames(arrayData) <- paste0("sample", 1:8)
#' group <- c(rep("group1", 4), rep("group2", 4))
#' names(group) <- colnames(arrayData)
#' result <- differential_array(df = arrayData, group = group)
differential_array <- function(df, group, method = "limma", 
                                adjust.method = "BH") {
    method <- match.arg(method, c("limma", "ttest", "wilcox"))
    samples <- intersect(colnames(df), names(group))
    group <- group[samples]
    df <- df[, samples]
    if (method == "limma") {
        result <- differential_limma(df, group, adjust.method = adjust.method)
    } else {
        groups <- unique(group)
        which1 <- which(group == groups[1])
        which2 <- which(group == groups[2])
        P.Value <- rep(0, nrow(df))
        if (method == "ttest") {
            for (i in seq_len(length(P.Value))) {
                P.Value[i] <- stats::t.test(df[i, which1],
                    df[i, which2])$p.value
            }
        } else {
            for (i in seq_len(length(P.Value))) {
                P.Value[i] <- stats::wilcox.test(as.numeric(df[i, which1]),
                    as.numeric(df[i, which2]))$p.value
            }
        }
        adj.P.Val <- stats::p.adjust(P.Value, method = adjust.method)
        result <- data.frame(gene = rownames(df),
            P.Value = P.Value, adj.P.Val = adj.P.Val)
    }
    return(result)
}


#' Get Microarray matrix data from GEO
#'
#' @param gse GSE number, such as GSE781.
#'
#' @return a list of matrix
#' @export
#'
#' @examples
#' \donttest{
#' arraylist <- get_geo_array("GSE781")
#' }
get_geo_array <- function(gse) {
    gse <- GEOquery::getGEO(gse, GSEMatrix = FALSE, AnnotGPL = TRUE)
    gselist <- vector("list", length(GEOquery::GPLList(gse)))
    names(gselist) <- names(GEOquery::GPLList(gse))
    gsmplatforms <- lapply(GEOquery::GSMList(gse),
        function(x) {GEOquery::Meta(x)$platform_id})
    for (i in seq_len(length(gselist))) {
        gsmlist <- BiocGenerics::Filter(function(gsm) {
            GEOquery::Meta(gsm)$platform_id==names(gselist)[i]},
        GEOquery::GSMList(gse))
        probesets <- GEOquery::Table(GEOquery::GPLList(gse)[[1]])$ID
        data.matrix <- do.call('cbind',
            lapply(gsmlist, function(x) {tab <- GEOquery::Table(x)
                                        mymatch <- match(probesets,tab$ID_REF)
                                        return(tab$VALUE[mymatch])
                }
            )
        )
        data.matrix <- apply(data.matrix,2,
            function(x) {as.numeric(as.character(x))})
        gpl <- gse@gpls[names(gselist)[i]]
        gpl <- gpl[[1]]@dataTable@table
        genes <- gpl[match(probesets, gpl[, "ID"]), "Gene Symbol"]
        rownames(data.matrix) <- genes
        colnames(data.matrix) <- names(gsmlist)
        gselist[[i]] <- data.matrix
    }
}

#' Preprocess of Microarray data
#'
#' @param x matrix of Microarray data, each column is a sample, 
#' and each row is a gene. 
#' @param missing_value Method to  impute missing expression data,
#' one of "zero" and "knn".
#' @param string a string, sep of the gene
#'
#' @return matrix
#' @export 
#'
#' @examples
#' \donttest{
#' arraylist <- get_geo_array("GSE781")
#' arraylist <- lapply(arraylist, array_preprocess)
#' }
array_preprocess <- function(x, missing_value = "knn", string = " /// ") {
    ## filter
    x <- x[!is.na(rownames(x)), ]
    x <- x[rownames(x) != "", ]
    aa <- rowSums(is.na(x))
    x <- x[aa < ncol(x)/2, ]

    ## impute missing
    if (missing_value == "zero") {
        x[is.na(x)] <- 0
    } else {
        x <- impute::impute.knn(x)$data
    }

    ## log
    qx <- as.numeric(stats::quantile(x, 
        c(0., 0.25, 0.5, 0.75, 0.99, 1.0), na.rm = TRUE))
    LogC <- (qx[5] > 100) || (qx[6]-qx[1] > 50 && qx[2] > 0)
    if (LogC) {
        x[which(x <= 0)] <- 0.0001
        x <- log2(x)
    }

    ## gene id conversion
    x <- cbind(rownames(x), x)
    x <- repAssign(x, string)
    # gene_ave(x)
}

#' cluster probes of Microarray data
#'
#' @param x matrix of Microarray data, the first is the name of the gene, 
#' and the others are the expression value.
#' @param clusterCutoff Pearson correlation threshold 
#' to cut off the hierarchical tree.
#' @importFrom stats as.dist
#' @importFrom stats cor
#' @importFrom stats cutree
#' @importFrom stats hclust
#' @return data.frame
#' @export 
#'
#' @examples
#' \donttest{
#' arraylist <- get_geo_array("GSE781")
#' arraylist <- lapply(arraylist, array_preprocess)
#' arraylist_cluster <- lapply(arraylist, cluster_array)
#' }
cluster_array <- function(x, clusterCutoff = 0.7) {
    genes <- x[, 1]
    uniqueGenes <- unique(genes)
    x <- x[, -1]
    matlist <- vector("list", length(uniqueGenes))
    for (i in seq_len(length(uniqueGenes))) {
        gene <- uniqueGenes[i]
        probes <- which(genes == gene)
        mat <-  x[probes, ]
        if (length(probes) == 1) {
            rownames(mat) <- gene
            matlist[[i]] <- mat
        } else {
            probeCorrelation <- cor(t(mat),method = 'pearson')
            ClusterResults <- hclust(as.dist(1-probeCorrelation), 
                method = "complete", members = NULL)
            #plot(ClusterResults)
            Clusters <- cutree(ClusterResults, h = clusterCutoff)
            clusterDf <- matrix(0, length(unique(Clusters)), ncol(mat)) |> 
                as.data.frame()
            for (j in seq_len(length(unique(Clusters)))) {
                tmpGeneProbes <-  which(Clusters == j)
                if (length(tmpGeneProbes) > 1) {
                    clusterDf[j, ] <- colMeans(mat[tmpGeneProbes,])
                } else {
                    clusterDf[j, ] <- mat[tmpGeneProbes,]
                }        
            }
            if (nrow(clusterDf) > 1) {
                rownames(clusterDf) <- 
                    paste(gene, seq(nrow(clusterDf)), sep = "_")
            } else {
                rownames(clusterDf) <- gene
            }
            
            colnames(clusterDf) <- colnames(mat)    
            matlist[[i]] <- clusterDf
        }
    }
    matlist <- do.call("rbind", matlist)
}

test_that("can parse example differential_array", {
    # skip_on_bioc()
    arrayData <- matrix(runif(200), 25, 8)
    rownames(arrayData) <- paste0("gene", 1:25)
    colnames(arrayData) <- paste0("sample", 1:8)
    group <- c(rep("group1", 4), rep("group2", 4))
    names(group) <- colnames(arrayData)
    result <- differential_array(df = arrayData, group = group)
    expect_true( "P.Value"    %in% colnames(result))
})

#' a data.frame of gene expression data
#'
#' It is a randomly generated expression data 
#' used as an example of functions in this package.
#' the rowname is gene symbols
#' the columns are gene expression values
#'
#' @format A data.frame with 10779 rows and 2 column
#'
"geneExpress"

# ' a matrix for Converting gene symbol to entrez_id or ensembl_gene_id
# '
# ' the hgnc data comes from HGNC website
# ' the columns represent "symbol", "locus_group", "locus_type", 
# ' "entrez_id" and "ensembl_gene_id"
# '
# ' @format A matrix with 37647 rows and 5 column
# '
"hgnc"

#' a matrix of gene expression data in TCGA
#'
#' It is a randomly generated expression data 
#' used as an example of functions in this package.
#' the first column represents the gene symbol
#'
#' the other columns represent the expression(FPKM) of genes
#'
#' @format A matrix with 10 rows and 10 column
#'
"profile"

#' a matrix of gene expression data in GEO
#'
#' It is a randomly generated expression data 
#' used as an example of functions in this package.
#' the first column represents the gene symbol
#'
#' the other columns represent the expression of genes
#'
#' @format A matrix with 32 rows and 20 column
#'
"ventricle"

#' a matrix of gene expression data in TCGA
#'
#' It is a randomly generated expression data 
#' used as an example of functions in this package.
#' the first column represents the gene symbol
#'
#' the other columns represent the expression(count) of genes
#'
#' @format A matrix with 100 rows and 150 column
#'
"kegg_liver"

#' a matrix of gene expression data in GEO
#'
#' the first column represents the gene symbol
#'
#' the other columns represent the expression of genes
#'
#' @format A matrix with 999 rows and 3 column
#'
"GSE66705_sample2"

#' a matrix of module name, gene symbols, and the number of gene symbols
#'
#' It is a randomly generated expression data 
#' used as an example of functions in this package.
#' @format A matrix with 176 rows and 3 column
#'
"module"


# ' a matrix for Converting gene symbol.
# ' 
# ' the hgnc data comes from HGNC website
# '
# ' @format A matrix with 43547 rows and 52 column
# '
"hgnc_file"


#' a data.frame of gene length and GC content
#'
#' the gene length and GC content data comes from 
#' TxDb.Hsapiens.UCSC.hg38.knownGene and
#' BSgenome.Hsapiens.UCSC.hg38
#'
#' @format A data.frame with 27341 rows and 2 column
#'
"gene_cov"

countToTpm_internal <- function(counts, effLen) {
    rate <- log(counts) - log(effLen)
    denom <- log(sum(exp(rate)))
    exp(rate - denom + log(1e6))
}
countToFpkm_internal <- function(counts, effLen) {
    N <- sum(counts)
    exp(log(counts) + log(1e9) - log(effLen) - log(N))
}

fpkmToTpm_internal <- function(fpkm) {
    exp(log(fpkm) - log(sum(fpkm)) + log(1e6))
}

countToEffCounts_internal <- function(counts, len, effLen) {
    counts * (len / effLen)
}

# if we have fpkm, then we can easily get the rate of counts/sum(counts).
# we can't get the real count value.
fpkmToCount_internal <- function(fpkm, effLen, N = 1e9) {
    # rate <- (fpkm * effLen)/10^9
    rate <- exp(log(fpkm) + log(effLen) - log(1e9))
    counts <- rate * N
}




#' Convert count to FPKM
#'
#' @param counts_matrix a matrix, colnames of counts_matrix are sample name,
#' rownames of counts_matrix are gene symbols
#' @param keyType keyType, one of keytypes(org.Hs.eg.db).
#' @param gene_cov data.frame of two column, the first column is gene length, 
#' the second column is gene GC content
#'
#' @return a matrix
#' @export
#'
#' @examples
#' data(gene_cov)
#' lung_squ_count2 <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), ncol = 3)
#' rownames(lung_squ_count2) <- c("DISC1", "TCOF1", "SPPL3")
#' colnames(lung_squ_count2) <- c("sample1", "sample2", "sample3")
#' result <- countToFpkm(lung_squ_count2,
#'     keyType = "SYMBOL",
#'     gene_cov = gene_cov
#' )
countToFpkm <- function(counts_matrix, keyType = "SYMBOL", gene_cov) {
    gene_cov2 <- gene_cov
    if (keyType != "ENTREZID") {
        genes_bitr <- clusterProfiler::bitr(rownames(gene_cov),
            fromType = "ENTREZID", toType = keyType,
            OrgDb = org.Hs.eg.db::org.Hs.eg.db, drop = TRUE
        )
        genes_bitr <- genes_bitr[!duplicated(genes_bitr[, 2]), ]
        gene_cov2 <- gene_cov[genes_bitr$ENTREZID, ]
        rownames(gene_cov2) <- genes_bitr[, 2]
    }
    genes_count <- intersect(rownames(counts_matrix), rownames(gene_cov2))
    counts_matrix_new <- counts_matrix[genes_count, ]
    gene_loc_len_new <- gene_cov2[genes_count, ]
    genes_length <- as.numeric(gene_loc_len_new[, 1])
    counts_matrix_new2 <- counts_matrix_new
    for (i in seq_len(dim(counts_matrix_new2)[2])) {
        counts_matrix_new2[, i] <- countToFpkm_internal(
            as.numeric(counts_matrix_new2[, i]),
            genes_length)
    }
    return(counts_matrix_new2)
}


#' Convert count to Tpm
#'
#' @param counts_matrix a matrix, colnames of counts_matrix are sample name,
#' rownames of counts_matrix are gene symbols
#' @param keyType keyType, one of keytypes(org.Hs.eg.db).
#' @param gene_cov data.frame of two column, the first column is gene length, 
#' the second column is gene GC content
#'
#' @return a matrix
#' @export
#'
#' @examples
#' data(gene_cov)
#' lung_squ_count2 <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), ncol = 3)
#' rownames(lung_squ_count2) <- c("DISC1", "TCOF1", "SPPL3")
#' colnames(lung_squ_count2) <- c("sample1", "sample2", "sample3")
#' result <- countToTpm(lung_squ_count2,
#'     keyType = "SYMBOL",
#'     gene_cov = gene_cov
#' )
countToTpm <- function(counts_matrix, keyType = "SYMBOL", gene_cov) {
    gene_cov2 <- gene_cov
    if (keyType != "ENTREZID") {
        genes_bitr <- clusterProfiler::bitr(rownames(gene_cov),
            fromType = "ENTREZID", toType = keyType,
            OrgDb = org.Hs.eg.db::org.Hs.eg.db, drop = TRUE
        )
        genes_bitr <- genes_bitr[!duplicated(genes_bitr[, 2]), ]
        gene_cov2 <- gene_cov[genes_bitr$ENTREZID, ]
        rownames(gene_cov2) <- genes_bitr[, 2]
    }
    genes_count <- intersect(rownames(counts_matrix), rownames(gene_cov2))
    counts_matrix_new <- counts_matrix[genes_count, ]
    gene_loc_len_new <- gene_cov2[genes_count, ]
    genes_length <- as.numeric(gene_loc_len_new[, 1])
    counts_matrix_new2 <- counts_matrix_new
    for (i in seq_len(dim(counts_matrix_new2)[2])) {
        counts_matrix_new2[, i] <- countToTpm_internal(
            as.numeric(counts_matrix_new2[, i]),
            genes_length)
    }
    return(counts_matrix_new2)
}


#' Convert fpkm to Tpm
#'
#' @param fpkm_matrix a matrix, colnames of fpkm_matrix are sample name,
#' rownames of fpkm_matrix are genes
#'
#' @return a matrix
#' @export
#'
#' @examples
#' lung_squ_count2 <- matrix(c(0.11, 0.22, 0.43, 0.14, 0.875,
#'     0.66, 0.77, 0.18, 0.29), ncol = 3)
#' rownames(lung_squ_count2) <- c("DISC1", "TCOF1", "SPPL3")
#' colnames(lung_squ_count2) <- c("sample1", "sample2", "sample3")
#' result <- fpkmToTpm(lung_squ_count2)
fpkmToTpm <- function(fpkm_matrix) {
    fpkm_matrix_new <- apply(fpkm_matrix, 2, fpkmToTpm_internal)
}

test_that("can parse example fpkmToTpm_matrix", {
    lung_squ_count2 <- matrix(c(0.11, 0.22, 0.43, 0.14, 0.875,
        0.66, 0.77, 0.18, 0.29), ncol = 3)
    rownames(lung_squ_count2) <- c("DISC1", "TCOF1", "SPPL3")
    colnames(lung_squ_count2) <- c("sample1", "sample2", "sample3")
    result <- fpkmToTpm(lung_squ_count2)
    expect_equal(dim(lung_squ_count2), dim(result))
})

test_that("can parse example countToFpkm_matrix", {
    data(gene_cov)
    lung_squ_count2 <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), ncol = 3)
    rownames(lung_squ_count2) <- c("DISC1", "TCOF1", "SPPL3")
    colnames(lung_squ_count2) <- c("sample1", "sample2", "sample3")
    result <- countToFpkm(lung_squ_count2,
        keyType = "SYMBOL",
        gene_cov = gene_cov
    )
    expect_equal(dim(lung_squ_count2), dim(result))
})

#' Get methylation difference gene
#'
#' @title differential_methy
#' @rdname differential_methy
#' @param cpgData data.frame of cpg beta value, , or SummarizedExperiment object
#' @param sampleGroup vector of sample group
#' @param groupCol group column
#' @param combineMethod method to combine the cpg pvalues, 
#' a function or one of "stouffer", "fisher" and "rhoScores".
#' @param missing_value Method to impute missing expression data,
#' one of "zero" and "knn".
#' @param cpg2gene data.frame to annotate cpg locus to gene
#' @param normMethod Method to do normalization: "PBC" or "BMIQ".
#' @param region region of genes, one of "Body", "TSS1500", "TSS200",
#' "3'UTR", "1stExon", "5'UTR", and "IGR". Only used when cpg2gene is NULL.
#' @param model if "cpg", step1: calculate difference cpgs;
#' step2: calculate difference genes.
#' if "gene", step1: calculate the methylation level of genes;
#' step2: calculate difference genes.
#' @param adjust.method character string specifying the method
#' used to adjust p-values for multiple testing.
#' See \link{p.adjust} for possible values.
#' @param ucscData Logical, whether the data comes from UCSC Xena.
#' @param adjPvalCutoff adjusted pvalue cutoff
#' @importFrom stats p.adjust
#' @return data.frame
#' @export
#' @examples
#' \donttest{
#' # use TCGAbiolinks data
#' library(TCGAbiolinks)
#' query <- GDCquery(project = "TCGA-ACC",
#'     data.category = "DNA Methylation",
#'     data.type = "Methylation Beta Value",
#'     platform = "Illumina Human Methylation 450")
#' GDCdownload(query, method = "api", files.per.chunk = 5,
#'     directory = Your_Path)
#' merge_result <- Merge_methy_tcga(Your_Path_to_DNA_Methylation_data)
#' library(ChAMP) # To avoid reporting errors
#' differential_gene <- differential_methy(cpgData = merge_result,
#'     sampleGroup = sample(c("C","T"),
#'     ncol(merge_result[[1]]), replace = TRUE))
#' }
#' # use user defined data
#' library(ChAMP)
#' cpgData <- matrix(runif(2000), nrow = 200, ncol = 10)
#' rownames(cpgData) <- paste0("cpg", seq_len(200))
#' colnames(cpgData) <- paste0("sample", seq_len(10))
#' sampleGroup <- c(rep("group1", 5), rep("group2", 5))
#' names(sampleGroup) <- colnames(cpgData)
#' cpg2gene <- data.frame(cpg = rownames(cpgData), 
#'     gene = rep(paste0("gene", seq_len(20)), 10))
#' result <- differential_methy(cpgData, sampleGroup, 
#'     cpg2gene = cpg2gene, normMethod = NULL)
#' # use SummarizedExperiment object input
#' library(ChAMP)
#' cpgData <- matrix(runif(2000), nrow = 200, ncol = 10)
#' rownames(cpgData) <- paste0("cpg", seq_len(200))
#' colnames(cpgData) <- paste0("sample", seq_len(10))
#' sampleGroup <- c(rep("group1", 5), rep("group2", 5))
#' names(sampleGroup) <- colnames(cpgData)
#' cpg2gene <- data.frame(cpg = rownames(cpgData), 
#'     gene = rep(paste0("gene", seq_len(20)), 10))
#' colData <- S4Vectors::DataFrame(
#'     row.names = colnames(cpgData),
#'     group = sampleGroup
#' )
#' data <- SummarizedExperiment::SummarizedExperiment(
#'          assays=S4Vectors::SimpleList(counts=cpgData),
#'          colData = colData)
#' result <- differential_methy(cpgData = data, 
#'     groupCol = "group", normMethod = NULL, 
#'     cpg2gene = cpg2gene)  
differential_methy  <- function(cpgData, sampleGroup,
                    groupCol,
                    # combineMethod = RobustRankAggreg::rhoScores,
                    combineMethod = "stouffer",
                    missing_value = "knn", 
                    cpg2gene = NULL,
                    normMethod = "PBC", 
                    region = "TSS1500",
                    model = "gene",
                    adjust.method = "BH",
                    adjPvalCutoff = 0.05,
                    ucscData = FALSE) {
    region <- match.arg(region, c("Body", "TSS1500", "TSS200",
        "3'UTR", "1stExon", "5'UTR", "IGR"))
    model <- match.arg(model, c("cpg", "gene"))

    if (inherits(cpgData,  "SummarizedExperiment")) {
        cpgData2 <- cpgData
        cpgData <- assays(cpgData2)$counts
        sampleGroup <- colData(cpgData2)[, groupCol]
        names(sampleGroup) <- rownames(colData(cpgData2))
    } else {
        if (inherits(cpgData,  "list")) { 
            cpgData <- cpgData[[1]]
        }
    }


    if (ucscData) {
        class(cpgData) <- "data.frame"
        rownames(cpgData) <- cpgData[, 1]
        cpgs <- rownames(cpgData)
        cpgData <- cpgData[, -1]
        group <- sampleGroup
        if (is.null(group)) {
            group <- lapply(colnames(cpgData), function(x) {
                strsplit(x, "-")[[1]][4]
            }) |> unlist()
    
            group <- substring(group, 1, 1)
        }
    }



    cpgData <- as.matrix(cpgData)
    # Use KNN to fill in missing values
    if (missing_value == "zero") {
        cpgData[is.na(cpgData)] <- 0
        data.m <- cpgData
    } else {
        data.m <- impute::impute.knn(cpgData)$data
    }

    # normalize data
    myNorm <- data.m
    if (!is.null(normMethod)) {
        myNorm <- ChAMP::champ.norm(beta = data.m, rgSet = NULL, 
            mset = NULL, method = normMethod)
    }
    if (!is.null(cpg2gene)) {
        cpg_gene <- cpg2gene
    } else {
        cpg_gene <- get_cpg_annotation(region = region)
    }


    if (model == "gene") {
        cpg_gene <-  split(cpg_gene[, 2], cpg_gene[, 1])   
        genes <- unlist(lapply(cpg_gene, function(x) {paste(x,collapse = ";")}))
        cpg_gene <- data.frame(cpg = names(cpg_gene), gene = genes)
        rownames(cpg_gene) <- cpg_gene[, 1]
        myNorm <- as.data.frame(myNorm)
        myNorm$gene <- cpg_gene[rownames(myNorm), 2]
        # myNorm <- myNorm[, c(ncol(myNorm), 1:(ncol(myNorm) - 1))]
        myNorm <- myNorm[, c(ncol(myNorm), seq_len(ncol(myNorm) - 1))]
        myNorm <- myNorm[!is.na(myNorm$gene), ]


        myNorm$gene <- as.character(myNorm$gene)
        myNorm2 <- repAssign(myNorm, ";")
        myNorm3 <- gene_ave(myNorm2)

        ## use limma to do differential expression analysis
        gene_pvalue <- differential_limma(myNorm3, group = sampleGroup,
            adjust.method = adjust.method)
        gene_pvalue$gene <- rownames(gene_pvalue)
    } else {
        # Identify Differential Methylation Positions (DMP)
        myDMP <- ChAMP::champ.DMP(beta = myNorm,
            pheno = sampleGroup, adjPVal = 1)
        myDMP <- as.data.frame(myDMP)

        # use cpg_gene to annotate CpGs
        pvalues <- cpg_gene
        pvalues$pvalue <- myDMP[cpg_gene[, 1], 4]
        # rownames(pvalues) <- pvalues[, 1]
        pvalues <- pvalues[!is.na(pvalues$pvalue), ]
        
        if (is.function(combineMethod)) {
            gene_pvalue <- stats::aggregate(pvalues[, 4],
                by = list(pvalues[, 2]),
                # FUN = combine_pvalue, combineMethod = combineMethod
                FUN = combineMethod
            )
            colnames(gene_pvalue) <- c("gene", "pvalue")
        } else {
            aa <- pvalues$pvalue
            bb <- split(aa, pvalues$gene)
            gene_pvalue <- data.frame(gene = names(bb), 
                pvalue = unlist(lapply(bb, function(x) x[1])))
            if (combineMethod == "stouffer") {
                
                myBetas <- myNorm[pvalues$cpg, ]
                myBetas <- split(as.data.frame(myBetas), pvalues$gene)
                correl <- lapply(myBetas, function(x) cor(t(x)))
                weights <- lapply(correl, function(x) 1/apply(x^2,1,sum))
                
                for (i in seq_len(nrow(gene_pvalue))) {
                    if (length(bb[[i]]) > 1) {
                        gene_pvalue[i, 2] <- sumz(bb[[i]], weights[[i]])$p
                    }       
                }
            }

            if (combineMethod == "fisher") {
                for (i in seq_len(nrow(gene_pvalue))) {
                    if (length(bb[[i]]) > 1) {
                        gene_pvalue[i, 2] <- sumlog(bb[[i]])$p
                    }       
                }
            }
        }
        


        # get logFC of genes
        myNorm2 <- myNorm[pvalues[, 1], ]
        myNorm2 <- stats::aggregate(myNorm2,
            by = list(pvalues[, 2]), FUN = mean)

        myNorm2 <- myNorm2[myNorm2[, 1] != "", ]
        rownames(myNorm2) <- myNorm2[, 1]
        myNorm2 <- myNorm2[, -1]
        groups <- sort(unique(sampleGroup))
        mean1 <- rowMeans(myNorm2[, sampleGroup == groups[1]], na.rm = TRUE)
        mean2 <- rowMeans(myNorm2[, sampleGroup == groups[2]], na.rm = TRUE)
        logFC <- mean1 - mean2            

        gene_pvalue$logFC <- logFC[gene_pvalue[, 1]]
        colnames(gene_pvalue) <- c("gene", "P.Value", "logFC")
        gene_pvalue$gene <- as.character(gene_pvalue$gene)
        gene_pvalue$adj.P.Val <- p.adjust(gene_pvalue$P.Value,
            method = adjust.method)
        rownames(gene_pvalue) <- gene_pvalue$gene
    }
    gene_pvalue <- gene_pvalue[gene_pvalue$adj.P.Val < adjPvalCutoff, ]
    return(gene_pvalue)                        
}



#' differential_limma
#'
#' @param df data.frame of the omic data
#' @param group a vector, group of samples.
#' @param adjust.method adjust.method.
#' @return data.frame
#' @export
#' @examples
#' df <- matrix(runif(200), 25, 8)
#' df <- as.data.frame(df)
#' rownames(df) <- paste0("gene", 1:25)
#' colnames(df) <- paste0("sample", 1:8)
#' group <- sample(c("group1", "group2"), 8, replace = TRUE)
#' result <- differential_limma(df = df, group = group)
differential_limma <- function(df, group, adjust.method = "BH") {
    groups <- unique(group)
    # if group is a numberic vector(even for c("0", "1")), will get errors.
    group <- gsub(groups[1], "nromal", group)
    group <- gsub(groups[2], "disease", group)
    design <- stats::model.matrix(~ 0 + factor(group))
    colnames(design) <- levels(factor(group))
    contrast.matrix <- limma::makeContrasts(
        contrasts = paste(colnames(design)[2:1],
        collapse = "-"
    ), levels = colnames(design))

    fit <- limma::lmFit(df, design)
    fit <- limma::contrasts.fit(fit, contrast.matrix)
    fit <- limma::eBayes(fit)
    limma::topTable(fit, adjust.method = adjust.method, number = Inf)
    ## or limma::topTable(fit, coef = 1, adjust='BH', number=Inf)
    ## contrasts.fit is not necessory
    # groups <- unique(group)
    # group <- gsub(groups[1], "nromal", group)
    # group <- gsub(groups[2], "disease", group)
    # design <- stats::model.matrix(~factor(group))

    # fit2 <- lmFit(df, design)
    # fit2 <- eBayes(fit2)
    # topTable(fit2,coef=2, adjust='BH', number=Inf)

    ## coef parameter is not necessory：
    # opTable(fit2, adjust='BH', number=Inf)
}

#' Merge methylation data downloaded from TCGA
#'
#' When the methylation data is downloaded from TCGA, 
#' each sample is saved in a folder, which contains the methylation value file 
#' and the descriptive file. This function can directly 
#' extract and consolidate all folders.
#' @param dirr a string for the directory of methylation data download from tcga
#' useing the tools gdc
#' @return a matrix, a combined methylation expression spectrum matrix
#' @export
#'
#' @examples
#' merge_result <- Merge_methy_tcga(system.file(file.path("extdata", "methy"),
#'     package = "GeoTcgaData"))
Merge_methy_tcga <- function(dirr = NULL) {
    options(warn = -1)
    # file_num=1
    if (is.null(dirr)) stop("please give your directory of methylation data!")
    tcga_dir <- dir(dirr)
    filePath <- file.path(dirr, tcga_dir[1])
    methyFile <- get_methy_df(filePath)
    methyResult <- matrix(0, nrow = nrow(methyFile), ncol = length(tcga_dir))
    rownames(methyResult) <- methyFile[, 1]
    samples <- rep(0, length(tcga_dir))
    methyResult[, 1] <- methyFile[, 2]
    samples[1] <- colnames(methyFile)[2]
    message("file", 1, " is over")
    for (i in 2:length(tcga_dir)) {
        message("file", i, " is over")
        filePath <- file.path(dirr, tcga_dir[i])
        methyFile <- get_methy_df(filePath)
        methyResult[, i] <- methyFile[, 2]
        samples[i] <- colnames(methyFile)[2]
        gc()
    }
    colnames(methyResult) <- samples
    cpg_info <- methyFile[, -2]
    return(list(methyResult = methyResult, cpg_info = cpg_info))
}

#' Read methylated data file and turn it into data frame
#'
#' @param filePath Path of files
#' @return data.frame
#' @noRd
get_methy_df <- function(filePath) {
    methyDir <- dir(filePath)
    if (length(methyDir) == 1) {
        file_name <- file.path(filePath, methyDir)
        sample <- unlist(strsplit(methyDir, "\\."))[1]
    } else {
        for (j in seq_len(length(methyDir))) {
            if (length(grep("jhu-usc", methyDir[j])) > 0) {
                file_name <- file.path(filePath, dir(filePath)[j])
                sample <- unlist(strsplit(dir(filePath)[j], "\\."))[6]
            }
        }
    }

    methyFile <- data.table::fread(file_name, header = FALSE)
    class(methyFile) <- "data.frame"
    colnames(methyFile)[2] <- sample
    return(methyFile)
}


get_cpg_annotation <- function(region = "TSS1500") {
    ## library to avoid errors.
    # library(IlluminaHumanMethylation450kanno.ilmn12.hg19)
    ann <- minfi::getAnnotation(
                IlluminaHumanMethylation450kanno.ilmn12.hg19::IlluminaHumanMethylation450kanno.ilmn12.hg19)
    ann <- as.data.frame(ann)
    cpg_gene <- ann[, c("Name", "UCSC_RefGene_Name", "UCSC_RefGene_Group")]
    # cpg_gene <- cpg_gene[grep(region, cpg_gene$UCSC_RefGene_Group), ]
    cpg_gene <- cpg_gene[cpg_gene[, 2] != "", ]
    genelist <- strsplit(cpg_gene[, 2], ";")
    regionlist <- strsplit(cpg_gene[, 3], ";")
    geneLength <- unlist(lapply(genelist, length))
    cpgs <- rep(cpg_gene[, 1], times = geneLength)
    cpg_gene2 <- data.frame(cpg = cpgs, gene = unlist(genelist), 
        region = unlist(regionlist))
    cpg_gene2 <- cpg_gene2[grep(region, cpg_gene2$region), ]
    return(unique(cpg_gene2))
}

test_that("can parse example Merge_methy_tcga", {
    merge_result <- Merge_methy_tcga(system.file(file.path("extdata", "methy"), 
      package = "GeoTcgaData"))
    expect_equal(names(merge_result), c("methyResult", "cpg_info"))
})

test_that("can parse example Diff_limma", {
    df <- matrix(runif(200), 25, 8)
    df <- as.data.frame(df)
    rownames(df) <- paste0("gene", 1:25)
    colnames(df) <- paste0("sample", 1:8)
    group <- sample(c("group1", "group2"), 8, replace = TRUE)
    result <- differential_limma(df = df, group = group)
    expect_true( "P.Value"  %in% colnames(result))
})

---
title: "GeoTcgaData"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{GeoTcgaData}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```
--------

## Authors
Erqiang Hu

Department of Bioinformatics, School of Basic Medical Sciences, 
Southern Medical University.


## Introduction
GEO and TCGA provide us with a wealth of data, such as RNA-seq, DNA Methylation,
single nucleotide Variation and Copy number variation data. 
It's easy to download data from TCGA using the  gdc tool or `TCGAbiolinks`, 
and some software provides organized TCGA data, such as 
[UCSC Xena](http://xena.ucsc.edu/) , UCSCXenaTools, and 
[sangerbox](http://vip.sangerbox.com/), but processing these data into a format
suitable for bioinformatics  analysis requires more work. This R package was 
developed to handle these data.

```{r setup}
library(GeoTcgaData)
```

## Example

This is a basic example which shows you how to solve a common problem:

### RNA-seq data differential expression analysis

It is convenient to use TCGAbiolinks  or 
[`GDCRNATools`](https://bioconductor.org/packages/GDCRNATools/) to download 
and analysis Gene expression data.  `TCGAbiolinks` use `edgeR` package to do 
differential expression analysis, while `GDCRNATools` can implement three most 
commonly used methods: limma, edgeR , and DESeq2 to identify differentially 
expressed  genes (DEGs).

Alicia Oshlack  et al. claimed that unlike the chip data, 
the RNA-seq data had one [bias](https://pubmed.ncbi.nlm.nih.gov/20132535/)[1]: 
the larger the transcript length / mean read count , the more likely it was to 
be  identified as a differential gene, 
while there was no such trend in the 
[chip data](https://pubmed.ncbi.nlm.nih.gov/19371405/)[2].


 However, when we use their chip data for difference analysis
 (using the limma package), we find that chip data has the same trend as 
 RNA-seq data. And we also found this trend in the difference analysis results 
 given by the data 
 [authors](https://genome.cshlp.org/content/18/9/1509.long)[3].

 

 It is worse noting that only technical replicate data, which has small gene 
 dispersions, shows this [bias](https://pubmed.ncbi.nlm.nih.gov/28545404/)[4]. 
 This is because in technical replicate RNA-seq data a long gene has more 
 reads mapping to it compared to a short gene of similar expression, 
 and most of the statistical methods used to detect differential expression
 have stronger detection ability for genes with more reads. However, we have 
 not deduced why there is such a bias in the current difference 
 analysis algorithms. 

Some software, such as [CQN](http://www.bioconductor.org/packages/cqn/) ,
present a 
[normalization algorithm](https://pubmed.ncbi.nlm.nih.gov/22285995/) [5]
to correct systematic biases(gene length bias and 
[GC-content bias](https://pubmed.ncbi.nlm.nih.gov/22177264/)[6]. 
But they did not provide sufficient evidence to prove that the correction is 
effective. We use the 
[Marioni dataset](https://pubmed.ncbi.nlm.nih.gov/19371405/)[2] to verify the
correction effect of CQN and find that there is still a deviation 
after correction:



[GOseq](http://bioconductor.org/packages/goseq/) [1]based on 
Wallenius' noncentral hypergeometric distribution can effectively correct the 
gene length deviation in enrichment analysis. However, the current RNA-seq data 
often have no gene length bias, but only the expression amount(read count) 
bias, GOseq may overcorrect these data, correcting originally unbiased data 
into reverse bias.

GOseq also fails to correct for expression bias, therefore, read count bias 
correction is still a challenge for us.

```{r, message=FALSE, warning=FALSE}
# use user-defined data
df <- matrix(rnbinom(400, mu = 4, size = 10), 25, 16)
df <- as.data.frame(df)
rownames(df) <- paste0("gene", 1:25)
colnames(df) <- paste0("sample", 1:16)
group <- sample(c("group1", "group2"), 16, replace = TRUE)
result <- differential_RNA(counts = df, group = group,
    filte = FALSE, method = "Wilcoxon")
# use SummarizedExperiment object input
df <- matrix(rnbinom(400, mu = 4, size = 10), 25, 16)
rownames(df) <- paste0("gene", 1:25)
colnames(df) <- paste0("sample", 1:16)
group <- sample(c("group1", "group2"), 16, replace = TRUE)

nrows <- 200; ncols <- 20
counts <- matrix(
    runif(nrows * ncols, 1, 1e4), nrows,
    dimnames = list(paste0("cg",1:200),paste0("S",1:20))
)

colData <- S4Vectors::DataFrame(
  row.names = paste0("sample", 1:16),
  group = group
)
data <- SummarizedExperiment::SummarizedExperiment(
         assays=S4Vectors::SimpleList(counts=df),
         colData = colData)

result <- differential_RNA(counts = data, groupCol = "group",
    filte = FALSE, method = "Wilcoxon") 
```


### DNA Methylation data integration 

use `TCGAbiolinks` data. 

The codes may need to be modified if `TCGAbiolinks` updates. 
So please read its documents.

```{r, message=FALSE, warning=FALSE}
# use user defined data
library(ChAMP)
cpgData <- matrix(runif(2000), nrow = 200, ncol = 10)
rownames(cpgData) <- paste0("cpg", seq_len(200))
colnames(cpgData) <- paste0("sample", seq_len(10))
sampleGroup <- c(rep("group1", 5), rep("group2", 5))
names(sampleGroup) <- colnames(cpgData)
cpg2gene <- data.frame(cpg = rownames(cpgData), 
    gene = rep(paste0("gene", seq_len(20)), 10))
result <- differential_methy(cpgData, sampleGroup, 
    cpg2gene = cpg2gene, normMethod = NULL)
# use SummarizedExperiment object input
library(ChAMP)
cpgData <- matrix(runif(2000), nrow = 200, ncol = 10)
rownames(cpgData) <- paste0("cpg", seq_len(200))
colnames(cpgData) <- paste0("sample", seq_len(10))
sampleGroup <- c(rep("group1", 5), rep("group2", 5))
names(sampleGroup) <- colnames(cpgData)
cpg2gene <- data.frame(cpg = rownames(cpgData), 
    gene = rep(paste0("gene", seq_len(20)), 10))
colData <- S4Vectors::DataFrame(
    row.names = colnames(cpgData),
    group = sampleGroup
)
data <- SummarizedExperiment::SummarizedExperiment(
         assays=S4Vectors::SimpleList(counts=cpgData),
         colData = colData)
result <- differential_methy(cpgData = data, 
    groupCol = "group", normMethod = NULL, 
    cpg2gene = cpg2gene)  
```
**Note:** `ChAMP`has a large number of dependent packages.
If you cannot install it  successfully, you can download each dependent package 
separately(Source or Binary) and install it  locally.

We provide two models to get methylation difference genes:  

if model = "cpg", step1: calculate difference cpgs; 
step2: calculate difference genes; 

if model = "gene", step1: calculate the methylation level of genes;
step2: calculate difference genes.

We find that only model = "gene" has no deviation of CpG number. 


### Copy number variation data integration and differential gene extraction

use TCGAbiolinks to download TCGA data(Gene Level Copy Number Scores)

```{r, message=FALSE, warning=FALSE}
# use random data as example
aa <- matrix(sample(c(0, 1, -1), 200, replace = TRUE), 25, 8)
rownames(aa) <- paste0("gene", 1:25)
colnames(aa) <- paste0("sample", 1:8)
sampleGroup <- sample(c("A", "B"), ncol(aa), replace = TRUE)
diffCnv <- differential_CNV(aa, sampleGroup)
```



### Difference analysis of single nucleotide Variation data 
We provide SNP_QC function to do quality control of SNP data
```{r, message=FALSE, warning=FALSE}
snpDf <- matrix(sample(c("AA", "Aa", "aa"), 100, replace = TRUE), 10, 10)
snpDf <- as.data.frame(snpDf)
sampleGroup <- sample(c("A", "B"), 10, replace = TRUE)
result <- SNP_QC(snpDf)
```

Then use differential_SNP to do differential analysis.
```{r, message=FALSE, warning=FALSE}
#' snpDf <- matrix(sample(c("mutation", NA), 100, replace = TRUE), 10, 10)
#' snpDf <- as.data.frame(snpDf)
#' sampleGroup <- sample(c("A", "B"), 10, replace = TRUE)
#' result <- differential_SNP(snpDf, sampleGroup)
```


### GEO chip data processing
The function `gene_ave` could average the expression data of different 
ids for the same gene in the GEO chip data. For example:

```{r, message=FALSE, warning=FALSE}
aa <- c("MARCH1","MARC1","MARCH1","MARCH1","MARCH1")
bb <- c(2.969058399,4.722410064,8.165514853,8.24243893,8.60815086)
cc <- c(3.969058399,5.722410064,7.165514853,6.24243893,7.60815086)
file_gene_ave <- data.frame(aa=aa,bb=bb,cc=cc)
colnames(file_gene_ave) <- c("Gene", "GSM1629982", "GSM1629983")
result <- gene_ave(file_gene_ave, 1)
```

Multiple genes symbols may correspond to a same chip id. The result of 
function `repAssign` is to assign the expression of this id to each gene, 
and function `repRemove` deletes the expression. For example:

```{r}
aa <- c("MARCH1 /// MMA","MARC1","MARCH2 /// MARCH3",
        "MARCH3 /// MARCH4","MARCH1")
bb <- c("2.969058399","4.722410064","8.165514853","8.24243893","8.60815086")
cc <- c("3.969058399","5.722410064","7.165514853","6.24243893","7.60815086")
input_file <- data.frame(aa=aa,bb=bb,cc=cc)
repAssign_result <- repAssign(input_file," /// ")
repRemove_result <- repRemove(input_file," /// ")
```

### Other downstream analyses

1. The function `id_conversion_TCGA` could convert  ENSEMBL gene id to 
gene Symbol in TCGA. For example:

```{r, message=FALSE, warning=FALSE}
data(profile)
result <- id_conversion_TCGA(profile)
```

The parameter `profile` is a data.frame or matrix of gene expression 
data in TCGA.

**Note:** In previous versions(< 1.0.0) the `id_conversion` and 
`id_conversion_TCGA` used HGNC data to convert human gene id.  
In future versions, we will use `clusterProfiler::bitr` for ID conversion. 


2. The function `countToFpkm` and `countToTpm` could convert 
count data to FPKM or TPM data.

```{r}
data(gene_cov)
lung_squ_count2 <- matrix(c(1,2,3,4,5,6,7,8,9),ncol=3)
rownames(lung_squ_count2) <- c("DISC1","TCOF1","SPPL3")
colnames(lung_squ_count2) <- c("sample1","sample2","sample3")
result <- countToFpkm(lung_squ_count2, keyType = "SYMBOL", 
    gene_cov = gene_cov)
```

```{r, message=FALSE, warning=FALSE}
data(gene_cov)
lung_squ_count2 <- matrix(c(0.11,0.22,0.43,0.14,0.875,
    0.66,0.77,0.18,0.29),ncol=3)
rownames(lung_squ_count2) <- c("DISC1","TCOF1","SPPL3")
colnames(lung_squ_count2) <- c("sample1","sample2","sample3")
result <- countToTpm(lung_squ_count2, keyType = "SYMBOL", 
    gene_cov = gene_cov)
```

```{r}
sessionInfo()
```

## References
1. Young MD, Wakefield MJ, Smyth GK, Oshlack A (2010) Gene ontology analysis 
for RNA-seq: accounting for selection bias. Genome Biol 11: R14.
2. Oshlack A, Wakefield MJ (2009) Transcript length bias in RNA-seq data 
confounds systems biology. Biol Direct 4: 14.
3. Marioni JC, Mason CE, Mane SM, Stephens M, Gilad Y (2008) RNA-seq: an 
assessment of technical reproducibility and comparison with gene expression 
arrays. Genome Res 18: 1509-1517.
4. Yoon S, Nam D (2017) Gene dispersion is the key determinant of the read 
count bias in differential expression analysis of RNA-seq data. 
BMC Genomics 18: 408.
5. Hansen KD, Irizarry RA, Wu Z (2012) Removing technical variability in 
RNA-seq data using conditional quantile normalization. 
Biostatistics 13: 204-216.
6. Risso D, Schwartz K, Sherlock G, Dudoit S (2011) GC-content normalization
for RNA-Seq data. BMC Bioinformatics 12: 480.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{geneExpress}
\alias{geneExpress}
\title{a data.frame of gene expression data}
\format{
A data.frame with 10779 rows and 2 column
}
\usage{
geneExpress
}
\description{
It is a randomly generated expression data 
used as an example of functions in this package.
the rowname is gene symbols
the columns are gene expression values
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gene_cov}
\alias{gene_cov}
\title{a data.frame of gene length and GC content}
\format{
A data.frame with 27341 rows and 2 column
}
\usage{
gene_cov
}
\description{
the gene length and GC content data comes from 
TxDb.Hsapiens.UCSC.hg38.knownGene and
BSgenome.Hsapiens.UCSC.hg38
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arrayDiff.R
\name{cluster_array}
\alias{cluster_array}
\title{cluster probes of Microarray data}
\usage{
cluster_array(x, clusterCutoff = 0.7)
}
\arguments{
\item{x}{matrix of Microarray data, the first is the name of the gene, 
and the others are the expression value.}

\item{clusterCutoff}{Pearson correlation threshold 
to cut off the hierarchical tree.}
}
\value{
data.frame
}
\description{
cluster probes of Microarray data
}
\examples{
\donttest{
arraylist <- get_geo_array("GSE781")
arraylist <- lapply(arraylist, array_preprocess)
arraylist_cluster <- lapply(arraylist, cluster_array)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SNP.R
\name{SNP_QC}
\alias{SNP_QC}
\title{Do quality control of SNP data downloaded from TCGAbiolinks}
\usage{
SNP_QC(
  snpData,
  geon = 0.02,
  mind = 0.02,
  maf = 0.05,
  hwe = 1e-06,
  miss = "NoCall"
)
}
\arguments{
\item{snpData}{data.frame of SNP data downloaded from TCGAbiolinks}

\item{geon}{filters out all variants with missing call rates
exceeding the provided value (default 0.02) to be removed}

\item{mind}{filters out all samples with missing call rates exceeding
the provided value (default 0.02) to be removed}

\item{maf}{filters out all variants with minor allele frequency below
the provided threshold}

\item{hwe}{filters out all variants which have Hardy-Weinberg
equilibrium exact test p-value below the provided threshold}

\item{miss}{character of miss value}
}
\value{
data.frame
}
\description{
Do quality control of SNP data downloaded from TCGAbiolinks
}
\examples{
# use demo data
snpDf <- matrix(sample(c("AA", "Aa", "aa"), 100, replace = TRUE), 10, 10)
snpDf <- as.data.frame(snpDf)
sampleGroup <- sample(c("A", "B"), 10, replace = TRUE)
result <- SNP_QC(snpDf)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Merge_methylation.R
\name{differential_methy}
\alias{differential_methy}
\title{differential_methy}
\usage{
differential_methy(
  cpgData,
  sampleGroup,
  groupCol,
  combineMethod = "stouffer",
  missing_value = "knn",
  cpg2gene = NULL,
  normMethod = "PBC",
  region = "TSS1500",
  model = "gene",
  adjust.method = "BH",
  adjPvalCutoff = 0.05,
  ucscData = FALSE
)
}
\arguments{
\item{cpgData}{data.frame of cpg beta value, , or SummarizedExperiment object}

\item{sampleGroup}{vector of sample group}

\item{groupCol}{group column}

\item{combineMethod}{method to combine the cpg pvalues, 
a function or one of "stouffer", "fisher" and "rhoScores".}

\item{missing_value}{Method to impute missing expression data,
one of "zero" and "knn".}

\item{cpg2gene}{data.frame to annotate cpg locus to gene}

\item{normMethod}{Method to do normalization: "PBC" or "BMIQ".}

\item{region}{region of genes, one of "Body", "TSS1500", "TSS200",
"3'UTR", "1stExon", "5'UTR", and "IGR". Only used when cpg2gene is NULL.}

\item{model}{if "cpg", step1: calculate difference cpgs;
step2: calculate difference genes.
if "gene", step1: calculate the methylation level of genes;
step2: calculate difference genes.}

\item{adjust.method}{character string specifying the method
used to adjust p-values for multiple testing.
See \link{p.adjust} for possible values.}

\item{adjPvalCutoff}{adjusted pvalue cutoff}

\item{ucscData}{Logical, whether the data comes from UCSC Xena.}
}
\value{
data.frame
}
\description{
Get methylation difference gene
}
\examples{
\donttest{
# use TCGAbiolinks data
library(TCGAbiolinks)
query <- GDCquery(project = "TCGA-ACC",
    data.category = "DNA Methylation",
    data.type = "Methylation Beta Value",
    platform = "Illumina Human Methylation 450")
GDCdownload(query, method = "api", files.per.chunk = 5,
    directory = Your_Path)
merge_result <- Merge_methy_tcga(Your_Path_to_DNA_Methylation_data)
library(ChAMP) # To avoid reporting errors
differential_gene <- differential_methy(cpgData = merge_result,
    sampleGroup = sample(c("C","T"),
    ncol(merge_result[[1]]), replace = TRUE))
}
# use user defined data
library(ChAMP)
cpgData <- matrix(runif(2000), nrow = 200, ncol = 10)
rownames(cpgData) <- paste0("cpg", seq_len(200))
colnames(cpgData) <- paste0("sample", seq_len(10))
sampleGroup <- c(rep("group1", 5), rep("group2", 5))
names(sampleGroup) <- colnames(cpgData)
cpg2gene <- data.frame(cpg = rownames(cpgData), 
    gene = rep(paste0("gene", seq_len(20)), 10))
result <- differential_methy(cpgData, sampleGroup, 
    cpg2gene = cpg2gene, normMethod = NULL)
# use SummarizedExperiment object input
library(ChAMP)
cpgData <- matrix(runif(2000), nrow = 200, ncol = 10)
rownames(cpgData) <- paste0("cpg", seq_len(200))
colnames(cpgData) <- paste0("sample", seq_len(10))
sampleGroup <- c(rep("group1", 5), rep("group2", 5))
names(sampleGroup) <- colnames(cpgData)
cpg2gene <- data.frame(cpg = rownames(cpgData), 
    gene = rep(paste0("gene", seq_len(20)), 10))
colData <- S4Vectors::DataFrame(
    row.names = colnames(cpgData),
    group = sampleGroup
)
data <- SummarizedExperiment::SummarizedExperiment(
         assays=S4Vectors::SimpleList(counts=cpgData),
         colData = colData)
result <- differential_methy(cpgData = data, 
    groupCol = "group", normMethod = NULL, 
    cpg2gene = cpg2gene)  
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SNP.R
\name{differential_SNP_GEO}
\alias{differential_SNP_GEO}
\title{Do difference analysis of SNP data downloaded from GEO}
\usage{
differential_SNP_GEO(snpData, sampleGroup, method = "Chisquare")
}
\arguments{
\item{snpData}{data.frame of SNP data downloaded from GEO}

\item{sampleGroup}{vector of sample group}

\item{method}{one of "Chisquare", "fisher",
and "CATT"(Cochran-Armitage trend test)}
}
\value{
data.frame
}
\description{
Do difference analysis of SNP data downloaded from GEO
}
\examples{
\donttest{
file1 <- read.table("GSE66903_series_matrix.txt.gz",
    fill=TRUE, comment.char="!", header = TRUE)
rownames(file1) <- file1[, 1]
snpData <- file1[, -1]
sampleGroup <- sample(c("A", "B"), ncol(snpData ), replace = TRUE)
names(sampleGroup) <- colnames(snpData)
snpData <- SNP_QC(snpData)
sampleGroup <- sample(c("A", "B"), ncol(snpData ), replace = TRUE)
result1 <- differential_SNP_GEO(snpData = snpData,
    sampleGroup = sampleGroup, method = "Chisquare")
}
# use demo data
snpDf <- matrix(sample(c("AA", "Aa", "aa"), 100, replace = TRUE), 10, 10)
snpDf <- as.data.frame(snpDf)
sampleGroup <- sample(c("A", "B"), 10, replace = TRUE)
result <- differential_SNP_GEO(snpDf, sampleGroup, method = "fisher")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rep.R
\name{repAssign}
\alias{repAssign}
\title{Handle the case where one id corresponds to multiple genes}
\usage{
repAssign(input_file, string)
}
\arguments{
\item{input_file}{input file, a data.frame or a matrix, 
the first column should be genes.}

\item{string}{a string, sep of the gene}
}
\value{
a data.frame, when an id corresponds to multiple genes,
the expression value is assigned to each gene
}
\description{
Handle the case where one id corresponds to multiple genes
}
\examples{
aa <- c("MARCH1 /// MMA", "MARC1", "MARCH2 /// MARCH3", 
    "MARCH3 /// MARCH4", "MARCH1")
bb <- c("2.969058399", "4.722410064", "8.165514853",
    "8.24243893", "8.60815086")
cc <- c("3.969058399", "5.722410064", "7.165514853", 
    "6.24243893", "7.60815086")
input_file <- data.frame(aa = aa, bb = bb, cc = cc)

repAssign_result <- repAssign(input_file, " /// ")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CNV.R
\name{differential_CNV}
\alias{differential_CNV}
\title{Do difference analysis of gene level copy number variation data}
\usage{
differential_CNV(
  cnvData,
  sampleGroup,
  method = "Chisquare",
  adjust.method = "BH",
  ...
)
}
\arguments{
\item{cnvData}{data.frame of CNV data, each column is a sample, 
and each row is a CNV.}

\item{sampleGroup}{vector of sample group}

\item{method}{method to do diffenenital analysis, 
one of "Chisquare", "fisher",
and "CATT"(Cochran-Armitage trend test)}

\item{adjust.method}{adjust.method, one of "holm", "hochberg", "hommel", 
"bonferroni", "BH", "BY", "fdr", and "none".}

\item{...}{parameters for "Chisquare", "fisher",
and "CATT"(Cochran-Armitage trend test)}
}
\value{
data.frame with pvalue and estimate
}
\description{
Do difference analysis of gene level copy number variation data
}
\examples{
\donttest{
# use TCGAbiolinks data as example
library(TCGAbiolinks)
query <- GDCquery(
        project = "TCGA-ACC",
        data.category = "Copy Number Variation",
        data.type = "Gene Level Copy Number",
        access = "open"
)
GDCdownload(query)
cnvData <- GDCprepare(query)
aa <- assays(cnvData)$copy_number
bb <- aa
aa[bb == 2] <- 0
aa[bb < 2] <- -1
aa[bb > 2] <- 1
sampleGroup <- sample(c("A", "B"), ncol(cnvData), replace = TRUE)
diffCnv <- differential_CNV(aa, sampleGroup)

# Use sangerbox CNV data as example
cnvData <- fread("Merge_GeneLevelCopyNumber.txt")
class(cnvData) <- "data.frame"
rownames(cnvData) <- cnvData[, 1]
cnvData <- cnvData[, -c(1, 2, 3)]
sampleGroup <- sample(c("A", "B"), ncol(cnvData), replace = TRUE)
diffCnv <- differential_CNV(cnvData, sampleGroup)
}
# use random data as example
aa <- matrix(sample(c(0, 1, -1), 200, replace = TRUE), 25, 8)
rownames(aa) <- paste0("gene", 1:25)
colnames(aa) <- paste0("sample", 1:8)
sampleGroup <- sample(c("A", "B"), ncol(aa), replace = TRUE)
diffCnv <- differential_CNV(aa, sampleGroup)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arrayDiff.R
\name{differential_array}
\alias{differential_array}
\title{Differential analysis of Microarray data}
\usage{
differential_array(df, group, method = "limma", adjust.method = "BH")
}
\arguments{
\item{df}{data.frame of the omic data, each column is a sample, 
and each row is a gene.}

\item{group}{a vector, group of samples.}

\item{method}{method to do differential analysis, 
one of "limma", "ttest", "wilcox".}

\item{adjust.method}{adjust.method, one of "holm", "hochberg", "hommel", 
"bonferroni", "BH", "BY", "fdr", and "none".}
}
\value{
data.frame
}
\description{
Differential analysis of Microarray data
}
\examples{
\donttest{
library(GeoTcgaData)
library(data.table)
# Use real GEO data as example
arrayData <- read.table("GSE54807_series_matrix.txt.gz",
    sep = "\t", header = TRUE,
        fill=TRUE, comment.char = "!", check.names=FALSE)
gpl <- fread("GPL6244-17930.txt", sep = "\t", header = TRUE)
gpl <- gpl[, c("ID", "gene_assignment")]
class(gpl) <- "data.frame"

for (i in seq_len(nrow(gpl))) {
        aa <- strsplit(gpl[i, 2], " // ")[[1]][5]
        gpl[i, 2] <- as.character(strsplit(aa, " /// ")[[1]][1])
}
gpl[,1] <- as.character(gpl[,1])
arrayData[, 1] <- as.character(arrayData[, 1])
rownames(gpl) <- gpl[, 1]
arrayData[, 1] <- gpl[arrayData[, 1], 2]


arrayData <- repRemove(arrayData," /// ")

# Remove rows that do not correspond to genes
arrayData <- arrayData[!is.na(arrayData[, 1]), ]
arrayData <- arrayData[!arrayData[, 1] == "", ]
arrayData <- arrayData[!arrayData[, 1] == "---", ]


arrayData <- arrayData[order(arrayData[, 1]), ]
arrayData <- gene_ave(arrayData, 1)

keep <- apply(arrayData, 1, function(x) sum(x < 1) < (length(x)/2))
arrayData <- arrayData[keep, ]

group <- c(rep("group1", 12), rep("group2", 12))
result <- differential_array(df = arrayData, group = group)
}
# Use random data as example
arrayData <- matrix(runif(200), 25, 8)
rownames(arrayData) <- paste0("gene", 1:25)
colnames(arrayData) <- paste0("sample", 1:8)
group <- c(rep("group1", 4), rep("group2", 4))
names(group) <- colnames(arrayData)
result <- differential_array(df = arrayData, group = group)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{module}
\alias{module}
\title{a matrix of module name, gene symbols, and the number of gene symbols}
\format{
A matrix with 176 rows and 3 column
}
\usage{
module
}
\description{
It is a randomly generated expression data 
used as an example of functions in this package.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{kegg_liver}
\alias{kegg_liver}
\title{a matrix of gene expression data in TCGA}
\format{
A matrix with 100 rows and 150 column
}
\usage{
kegg_liver
}
\description{
It is a randomly generated expression data 
used as an example of functions in this package.
the first column represents the gene symbol
}
\details{
the other columns represent the expression(count) of genes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SNP.R
\name{differential_SNP_tcga}
\alias{differential_SNP_tcga}
\title{Do difference analysis of SNP data downloaded from TCGAbiolinks}
\usage{
differential_SNP_tcga(snpData, sampleGroup, combineMethod = NULL)
}
\arguments{
\item{snpData}{data.frame of SNP data downloaded from TCGAbiolinks}

\item{sampleGroup}{vector of sample group}

\item{combineMethod}{Method of combining the pvalue of
multiple snp in a gene.}
}
\value{
data.frame
}
\description{
Do difference analysis of SNP data downloaded from TCGAbiolinks
}
\examples{
\donttest{
library(TCGAbiolinks)
query <- GDCquery(
    project = "TCGA-CHOL",
    data.category = "Simple Nucleotide Variation",
    access = "open",
    legacy = FALSE,
    data.type = "Masked Somatic Mutation",
    workflow.type = "Aliquot Ensemble Somatic Variant Merging and Masking"
)
GDCdownload(query)
data_snp <- GDCprepare(query)
samples <- unique(data_snp$Tumor_Sample_Barcode)
sampleGroup <- sample(c("A", "B"), length(samples), replace = TRUE)
names(sampleGroup) <- samples
pvalue <- differential_SNP_tcga(snpData = data_snp, 
    sampleGroup = sampleGroup)
}
# use demo data
snpDf <- matrix(sample(c("mutation", NA), 100, replace = TRUE), 10, 10)
snpDf <- as.data.frame(snpDf)
sampleGroup <- sample(c("A", "B"), 10, replace = TRUE)
result <- differential_SNP(snpDf, sampleGroup)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fpkm_count_conversion.r
\name{fpkmToTpm}
\alias{fpkmToTpm}
\title{Convert fpkm to Tpm}
\usage{
fpkmToTpm(fpkm_matrix)
}
\arguments{
\item{fpkm_matrix}{a matrix, colnames of fpkm_matrix are sample name,
rownames of fpkm_matrix are genes}
}
\value{
a matrix
}
\description{
Convert fpkm to Tpm
}
\examples{
lung_squ_count2 <- matrix(c(0.11, 0.22, 0.43, 0.14, 0.875,
    0.66, 0.77, 0.18, 0.29), ncol = 3)
rownames(lung_squ_count2) <- c("DISC1", "TCOF1", "SPPL3")
colnames(lung_squ_count2) <- c("sample1", "sample2", "sample3")
result <- fpkmToTpm(lung_squ_count2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepareChi.r
\name{prepare_chi}
\alias{prepare_chi}
\title{Preparer file for chi-square test}
\usage{
prepare_chi(cnv)
}
\arguments{
\item{cnv}{result of ann_merge()}
}
\value{
a matrix
}
\description{
Preparer file for chi-square test
}
\examples{
cnv <- matrix(c(
    -1.09150, -1.47120, -0.87050, -0.50880,
    -0.50880, 2.0, 2.0, 2.0, 2.0, 2.0, 2.601962, 2.621332, 2.621332,
    2.621332, 2.621332, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
    2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0
), nrow = 5)
cnv <- as.data.frame(cnv)
rownames(cnv) <- c("AJAP1", "FHAD1", "CLCNKB", "CROCCP2", "AL137798.3")
colnames(cnv) <- c(
    "TCGA-DD-A4NS-10A-01D-A30U-01", "TCGA-ED-A82E-01A-11D-A34Y-01",
    "TCGA-WQ-A9G7-01A-11D-A36W-01", "TCGA-DD-AADN-01A-11D-A40Q-01",
    "TCGA-ZS-A9CD-10A-01D-A36Z-01", "TCGA-DD-A1EB-11A-11D-A12Y-01"
)
cnv_chi_file <- prepare_chi(cnv)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_mean_profile.R
\name{gene_ave}
\alias{gene_ave}
\title{Average the values of same genes in gene expression profile}
\usage{
gene_ave(file_gene_ave, k = 1)
}
\arguments{
\item{file_gene_ave}{a data.frame of gene expression data, 
each column is a sample, and each row is a gene.}

\item{k}{a number, indicates which is the gene column.}
}
\value{
a data.frame, the values of same genes in gene expression profile
}
\description{
Average the values of same genes in gene expression profile
}
\examples{
aa <- c("MARCH1", "MARC1", "MARCH1", "MARCH1", "MARCH1")
bb <- c(2.969058399, 4.722410064, 8.165514853, 8.24243893, 8.60815086)
cc <- c(3.969058399, 5.722410064, 7.165514853, 6.24243893, 7.60815086)
file_gene_ave <- data.frame(aa = aa, bb = bb, cc = cc)
colnames(file_gene_ave) <- c("Gene", "GSM1629982", "GSM1629983")

result <- gene_ave(file_gene_ave, 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arrayDiff.R
\name{array_preprocess}
\alias{array_preprocess}
\title{Preprocess of Microarray data}
\usage{
array_preprocess(x, missing_value = "knn", string = " /// ")
}
\arguments{
\item{x}{matrix of Microarray data, each column is a sample, 
and each row is a gene.}

\item{missing_value}{Method to  impute missing expression data,
one of "zero" and "knn".}

\item{string}{a string, sep of the gene}
}
\value{
matrix
}
\description{
Preprocess of Microarray data
}
\examples{
\donttest{
arraylist <- get_geo_array("GSE781")
arraylist <- lapply(arraylist, array_preprocess)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Merge_methylation.R
\name{Merge_methy_tcga}
\alias{Merge_methy_tcga}
\title{Merge methylation data downloaded from TCGA}
\usage{
Merge_methy_tcga(dirr = NULL)
}
\arguments{
\item{dirr}{a string for the directory of methylation data download from tcga
useing the tools gdc}
}
\value{
a matrix, a combined methylation expression spectrum matrix
}
\description{
When the methylation data is downloaded from TCGA, 
each sample is saved in a folder, which contains the methylation value file 
and the descriptive file. This function can directly 
extract and consolidate all folders.
}
\examples{
merge_result <- Merge_methy_tcga(system.file(file.path("extdata", "methy"),
    package = "GeoTcgaData"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RNA_seq.R
\name{differential_RNA}
\alias{differential_RNA}
\title{differential_RNA}
\usage{
differential_RNA(
  counts,
  group,
  groupCol,
  method = "limma",
  geneLength = NULL,
  gccontent = NULL,
  filter = TRUE,
  edgeRNorm = TRUE,
  adjust.method = "BH",
  useTopconfects = TRUE,
  ucscData = FALSE
)
}
\arguments{
\item{counts}{a dataframe or numeric matrix of raw counts data, 
or SummarizedExperiment object}

\item{group}{sample groups}

\item{groupCol}{group column}

\item{method}{one of "DESeq2", "edgeR" , "limma", "dearseq",
"NOISeq", "Wilcoxon", and "auto".}

\item{geneLength}{a vector of gene length.}

\item{gccontent}{a vector of gene GC content.}

\item{filter}{if TRUE, use filterByExpr to filter genes.}

\item{edgeRNorm}{if TRUE, use edgeR to do normalization for dearseq method.}

\item{adjust.method}{character string specifying the method used to
adjust p-values for multiple testing.
See \link{p.adjust} for possible values.}

\item{useTopconfects}{if TRUE, use topconfects to provide a
more biologically useful ranked gene list.}

\item{ucscData}{Logical, whether the data comes from UCSC Xena.}
}
\value{
data.frame
}
\description{
Do difference analysis of RNA-seq data
}
\examples{
\donttest{
library(TCGAbiolinks)

query <- GDCquery(
    project = "TCGA-ACC",
    data.category = "Transcriptome Profiling",
    data.type = "Gene Expression Quantification",
    workflow.type = "STAR - Counts"
)

GDCdownload(query,
    method = "api", files.per.chunk = 3,
    directory = Your_Path
)

dataRNA <- GDCprepare(
    query = query, directory = Your_Path,
    save = TRUE, save.filename = "dataRNA.RData"
)
## get raw count matrix
dataPrep <- TCGAanalyze_Preprocessing(
    object = dataRNA,
    cor.cut = 0.6,
    datatype = "STAR - Counts"
)

# Use `differential_RNA` to do difference analysis.
# We provide the data of human gene length and GC content in `gene_cov`.
group <- sample(c("grp1", "grp2"), ncol(dataPrep), replace = TRUE)
library(cqn) # To avoid reporting errors: there is no function "rq"
## get gene length and GC content
library(org.Hs.eg.db)
genes_bitr <- bitr(rownames(gene_cov),
    fromType = "ENTREZID", toType = "ENSEMBL",
    OrgDb = org.Hs.eg.db, drop = TRUE
)
genes_bitr <- genes_bitr[!duplicated(genes_bitr[, 2]), ]
gene_cov2 <- gene_cov[genes_bitr$ENTREZID, ]
rownames(gene_cov2) <- genes_bitr$ENSEMBL
genes <- intersect(rownames(dataPrep), rownames(gene_cov2))
dataPrep <- dataPrep[genes, ]
geneLength <- gene_cov2(genes, "length")
gccontent <- gene_cov2(genes, "GC")
names(geneLength) <- names(gccontent) <- genes
##    Difference analysis
DEGAll <- differential_RNA(
    counts = dataPrep, group = group,
    geneLength = geneLength, gccontent = gccontent
)
# Use `clusterProfiler` to do enrichment analytics:
diffGenes <- DEGAll$logFC
names(diffGenes) <- rownames(DEGAll)
diffGenes <- sort(diffGenes, decreasing = TRUE)
library(clusterProfiler)
library(enrichplot)
library(org.Hs.eg.db)
gsego <- gseGO(gene = diffGenes, OrgDb = org.Hs.eg.db, keyType = "ENSEMBL")
dotplot(gsego)
}
# use user-defined data
df <- matrix(rnbinom(400, mu = 4, size = 10), 25, 16)
df <- as.data.frame(df)
rownames(df) <- paste0("gene", 1:25)
colnames(df) <- paste0("sample", 1:16)
group <- sample(c("group1", "group2"), 16, replace = TRUE)
result <- differential_RNA(counts = df, group = group,
    filte = FALSE, method = "Wilcoxon")
# use SummarizedExperiment object input
df <- matrix(rnbinom(400, mu = 4, size = 10), 25, 16)
rownames(df) <- paste0("gene", 1:25)
colnames(df) <- paste0("sample", 1:16)
group <- sample(c("group1", "group2"), 16, replace = TRUE)

nrows <- 200; ncols <- 20
 counts <- matrix(
   runif(nrows * ncols, 1, 1e4), nrows,
   dimnames = list(paste0("cg",1:200),paste0("S",1:20))
)

colData <- S4Vectors::DataFrame(
  row.names = paste0("sample", 1:16),
  group = group
)
data <- SummarizedExperiment::SummarizedExperiment(
         assays=S4Vectors::SimpleList(counts=df),
         colData = colData)

result <- differential_RNA(counts = data, groupCol = "group",
    filte = FALSE, method = "Wilcoxon") 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TCGA_id_conversion.R
\name{id_conversion_TCGA}
\alias{id_conversion_TCGA}
\title{Convert ENSEMBL gene id to gene Symbol in TCGA}
\usage{
id_conversion_TCGA(profiles, toType = "SYMBOL")
}
\arguments{
\item{profiles}{a data.frame of gene expression data, 
each column is a sample, 
and each row is a gene.}

\item{toType}{one of 'keytypes(org.Hs.eg.db)'}
}
\value{
a data.frame, gene symbols and their expression value
}
\description{
Convert ENSEMBL gene id to gene Symbol in TCGA
}
\examples{
library(org.Hs.eg.db)
data(profile)
result <- id_conversion_TCGA(profile)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Merge_methylation.R
\name{differential_limma}
\alias{differential_limma}
\title{differential_limma}
\usage{
differential_limma(df, group, adjust.method = "BH")
}
\arguments{
\item{df}{data.frame of the omic data}

\item{group}{a vector, group of samples.}

\item{adjust.method}{adjust.method.}
}
\value{
data.frame
}
\description{
differential_limma
}
\examples{
df <- matrix(runif(200), 25, 8)
df <- as.data.frame(df)
rownames(df) <- paste0("gene", 1:25)
colnames(df) <- paste0("sample", 1:8)
group <- sample(c("group1", "group2"), 8, replace = TRUE)
result <- differential_limma(df = df, group = group)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fpkm_count_conversion.r
\name{countToTpm}
\alias{countToTpm}
\title{Convert count to Tpm}
\usage{
countToTpm(counts_matrix, keyType = "SYMBOL", gene_cov)
}
\arguments{
\item{counts_matrix}{a matrix, colnames of counts_matrix are sample name,
rownames of counts_matrix are gene symbols}

\item{keyType}{keyType, one of keytypes(org.Hs.eg.db).}

\item{gene_cov}{data.frame of two column, the first column is gene length, 
the second column is gene GC content}
}
\value{
a matrix
}
\description{
Convert count to Tpm
}
\examples{
data(gene_cov)
lung_squ_count2 <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), ncol = 3)
rownames(lung_squ_count2) <- c("DISC1", "TCOF1", "SPPL3")
colnames(lung_squ_count2) <- c("sample1", "sample2", "sample3")
result <- countToTpm(lung_squ_count2,
    keyType = "SYMBOL",
    gene_cov = gene_cov
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arrayDiff.R
\name{get_geo_array}
\alias{get_geo_array}
\title{Get Microarray matrix data from GEO}
\usage{
get_geo_array(gse)
}
\arguments{
\item{gse}{GSE number, such as GSE781.}
}
\value{
a list of matrix
}
\description{
Get Microarray matrix data from GEO
}
\examples{
\donttest{
arraylist <- get_geo_array("GSE781")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SNP.R
\name{combine_pvalue}
\alias{combine_pvalue}
\title{combine pvalues of SNP difference analysis result}
\usage{
combine_pvalue(snpResult, snp2gene, combineMethod = min)
}
\arguments{
\item{snpResult}{data.frame of SNP difference analysis result.}

\item{snp2gene}{data frame of two column: snp and gene.}

\item{combineMethod}{Method of combining the
pvalue of multiple snp in a gene.}
}
\value{
data.frame
}
\description{
combine pvalues of SNP difference analysis result
}
\examples{
snpResult <- data.frame(pvalue = runif(100), estimate = runif(100))
rownames(snpResult) <- paste0("snp", seq_len(100))
snp2gene <- data.frame(snp = rownames(snpResult), 
    gene = rep(paste0("gene", seq_len(20)), 5))
result <- combine_pvalue(snpResult, snp2gene)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_mean_module.R
\name{cal_mean_module}
\alias{cal_mean_module}
\title{Find the mean value of the gene in each module}
\usage{
cal_mean_module(geneExpress, module)
}
\arguments{
\item{geneExpress}{a data.frame of gene expression data. 
Each column is a sample, and each row is a gene.}

\item{module}{a data.frame of two column. The first column is module name, 
the second column are genes in this module.}
}
\value{
a data.frame, means the mean of gene expression value in
the same module
}
\description{
Find the mean value of the gene in each module
}
\examples{
data(geneExpress)
data(module)
result <- cal_mean_module(geneExpress, module)
}
